@@ -190,7 +190,8 @@ enum {
 	ESLURM_STEP_LIMIT,
 	ESLURM_JOB_SUSPENDED,
 	ESLURM_CAN_NOT_START_IMMEDIATELY,
-
+	ESLURM_INTERCONNECT_BUSY,
+	ESLURM_RESERVATION_EMPTY,
 	/* switch specific error codes, specific values defined in plugin module */
 	ESLURM_SWITCH_MIN = 3000,
 	ESLURM_SWITCH_MAX = 3099,
