@@ -192,6 +192,18 @@ extern void *slurm_ctl_conf_2_key_pairs (slurm_ctl_conf_t* slurm_ctl_conf_ptr)
 		key_pair->value = xstrdup("NO");
 	list_append(ret_list, key_pair);
 
+	key_pair = xmalloc(sizeof(config_key_pair_t));
+	key_pair->name = xstrdup("AcctGatherEnergyType");
+	key_pair->value = xstrdup(slurm_ctl_conf_ptr->acct_gather_energy_type);
+	list_append(ret_list, key_pair);
+
+	snprintf(tmp_str, sizeof(tmp_str), "%u sec",
+		 slurm_ctl_conf_ptr->acct_gather_node_freq);
+	key_pair = xmalloc(sizeof(config_key_pair_t));
+	key_pair->name = xstrdup("AcctGatherNodeFreq");
+	key_pair->value = xstrdup(tmp_str);
+	list_append(ret_list, key_pair);
+
 	key_pair = xmalloc(sizeof(config_key_pair_t));
 	key_pair->name = xstrdup("AuthType");
 	key_pair->value = xstrdup(slurm_ctl_conf_ptr->authtype);
@@ -489,11 +501,21 @@ extern void *slurm_ctl_conf_2_key_pairs (slurm_ctl_conf_t* slurm_ctl_conf_ptr)
 	key_pair->value = xstrdup(tmp_str);
 	list_append(ret_list, key_pair);
 
+	key_pair = xmalloc(sizeof(config_key_pair_t));
+	key_pair->name = xstrdup("LaunchType");
+	key_pair->value = xstrdup(slurm_ctl_conf_ptr->launch_type);
+	list_append(ret_list, key_pair);
+
 	key_pair = xmalloc(sizeof(config_key_pair_t));
 	key_pair->name = xstrdup("Licenses");
 	key_pair->value = xstrdup(slurm_ctl_conf_ptr->licenses);
 	list_append(ret_list, key_pair);
 
+	key_pair = xmalloc(sizeof(config_key_pair_t));
+	key_pair->name = xstrdup("LicensesUsed");
+	key_pair->value = xstrdup(slurm_ctl_conf_ptr->licenses_used);
+	list_append(ret_list, key_pair);
+
 	key_pair = xmalloc(sizeof(config_key_pair_t));
 	key_pair->name = xstrdup("MailProg");
 	key_pair->value = xstrdup(slurm_ctl_conf_ptr->mail_prog);
@@ -1233,6 +1255,8 @@ void slurm_print_slurmd_status (FILE* out,
 
 	fprintf(out, "Actual CPUs              = %u\n",
 		slurmd_status_ptr->actual_cpus);
+	fprintf(out, "Actual Boards            = %u\n",
+		slurmd_status_ptr->actual_boards);
 	fprintf(out, "Actual sockets           = %u\n",
 		slurmd_status_ptr->actual_sockets);
 	fprintf(out, "Actual cores             = %u\n",
