@@ -53,7 +53,7 @@
  *  CODE-OCEC-09-009. All rights reserved.
  *
  *  This file is part of SLURM, a resource management program.
- *  For details, see <https://computing.llnl.gov/linux/slurm/>.
+ *  For details, see <http://www.schedmd.com/slurmdocs/>.
  *  Please also read the included file: DISCLAIMER.
  *
  *  SLURM is free software; you can redistribute it and/or modify it under
@@ -90,9 +90,10 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <slurm/pmi.h>
-#include <slurm/slurm.h>
-#include <slurm/slurm_errno.h>
+
+#include "slurm/pmi.h"
+#include "slurm/slurm.h"
+#include "slurm/slurm_errno.h"
 
 #include "src/api/slurm_pmi.h"
 #include "src/common/macros.h"
