@@ -58,6 +58,7 @@
 #include "src/common/bitstring.h"
 #include "src/common/hostlist.h"
 #include "src/common/net.h"
+#include "src/common/read_config.h"
 #include "src/common/slurm_cred.h"
 #include "src/common/slurm_protocol_api.h"
 #include "src/common/slurm_protocol_defs.h"
@@ -108,6 +109,7 @@ static job_step_create_request_msg_t *_create_step_request(
 	step_req->min_nodes = step_params->min_nodes;
 	step_req->max_nodes = step_params->max_nodes;
 	step_req->cpu_count = step_params->cpu_count;
+	step_req->cpu_freq  = step_params->cpu_freq;
 	step_req->num_tasks = step_params->task_count;
 	step_req->relative = step_params->relative;
 	step_req->resv_port_cnt = step_params->resv_port_cnt;
@@ -205,7 +207,6 @@ slurm_step_ctx_create_no_alloc (const slurm_step_ctx_params_t *step_params,
 	int sock = -1;
 	short port = 0;
 	int errnum = 0;
-	int cyclic = (step_params->task_dist == SLURM_DIST_CYCLIC);
 
 	/* First copy the user's step_params into a step request struct */
 	step_req = _create_step_request(step_params);
@@ -238,7 +239,8 @@ slurm_step_ctx_create_no_alloc (const slurm_step_ctx_params_t *step_params,
 	if (switch_build_jobinfo(step_resp->switch_job,
 				 step_resp->step_layout->node_list,
 				 step_resp->step_layout->tasks,
-				 cyclic, step_req->network) < 0)
+				 step_resp->step_layout->tids,
+				 step_req->network) < 0)
 		fatal("switch_build_jobinfo: %m");
 
 
@@ -348,6 +350,7 @@ slurm_step_ctx_get (slurm_step_ctx_t *ctx, int ctx_key, ...)
 	case SLURM_STEP_CTX_USER_MANAGED_SOCKETS:
 		int_ptr = va_arg(ap, int *);
 		int_array_pptr = va_arg(ap, int **);
+
 		if (ctx->launch_state == NULL
 		    || ctx->launch_state->user_managed_io == false
 		    || ctx->launch_state->io.user == NULL) {
@@ -411,50 +414,64 @@ slurm_step_ctx_destroy (slurm_step_ctx_t *ctx)
  *	to run a single process per node, regardless of the settings
  *	selected at slurm_step_ctx_create time.
  *
- *	This is primarily used on AIX by the slurm_ll_api in support of
- * 	poe.  The slurm_ll_api will want to launch a single pmd daemon
+ *	This is primarily used when launching 1 task per node as done
+ * 	with IBM's PE where we want to launch a single pmd daemon
  *	on each node regardless of the number of tasks running on each
  *	node.
  * IN ctx - job step context generated by slurm_step_ctx_create
+ * IN node_list - node list of nodes to run on
+ * IN node_cnt - number of nodes to run on
+ * IN/OUT curr_task_num - task_id of newest task, initialze to zero
  * RET SLURM_SUCCESS or SLURM_ERROR (with slurm_errno set)
  */
 extern int
-slurm_step_ctx_daemon_per_node_hack(slurm_step_ctx_t *ctx)
+slurm_step_ctx_daemon_per_node_hack(
+	slurm_step_ctx_t *ctx, char *node_list, uint32_t node_cnt,
+	uint32_t *curr_task_num)
 {
-	slurm_step_layout_t *new_layout, *old_layout;
-	int i;
+	slurm_step_layout_t *layout;
+	int i, orig_task_num = *curr_task_num;
+	int sock = -1;
 
 	if ((ctx == NULL) || (ctx->magic != STEP_CTX_MAGIC)) {
 		slurm_seterrno(EINVAL);
 		return SLURM_ERROR;
 	}
 
-	/* hack the context node count */
-	ctx->step_req->num_tasks = ctx->step_req->min_nodes;
+	if (!orig_task_num) {
+		/* hack the context step layout */
+		sock = ctx->launch_state->slurmctld_socket_fd;
+		slurm_step_layout_destroy(ctx->step_resp->step_layout);
+		ctx->step_resp->step_layout =
+			xmalloc(sizeof(slurm_step_layout_t));
+		layout = ctx->step_resp->step_layout;
 
-	/* hack the context step layout */
-	old_layout = ctx->step_resp->step_layout;
-	new_layout = (slurm_step_layout_t *)
-		     xmalloc(sizeof(slurm_step_layout_t));
-	new_layout->node_cnt = old_layout->node_cnt;
-	new_layout->task_cnt = old_layout->node_cnt;
-	new_layout->node_list = xstrdup(old_layout->node_list);
-	slurm_step_layout_destroy(old_layout);
-	new_layout->tasks = (uint16_t *) xmalloc(sizeof(uint16_t) *
-						 new_layout->node_cnt);
-	new_layout->tids = (uint32_t **) xmalloc(sizeof(uint32_t *) *
-						 new_layout->node_cnt);
-	for (i = 0; i < new_layout->node_cnt; i++) {
-		new_layout->tasks[i] = 1;
-		new_layout->tids[i] = (uint32_t *)xmalloc(sizeof(uint32_t));
-		new_layout->tids[i][0] = i;
+		layout->tasks = xmalloc(sizeof(uint16_t) * node_cnt);
+		layout->tids = xmalloc(sizeof(uint32_t *) * node_cnt);
+	} else {
+		layout = ctx->step_resp->step_layout;
+		xrealloc(layout->tasks, sizeof(uint16_t) * node_cnt);
+		xrealloc(layout->tids, sizeof(uint32_t *) * node_cnt);
 	}
-	ctx->step_resp->step_layout = new_layout;
 
-	/* recreate the launch state structure now that the settings
-	   have changed */
-	step_launch_state_destroy(ctx->launch_state);
-	ctx->launch_state = step_launch_state_create(ctx);
+	ctx->step_req->num_tasks = layout->task_cnt = layout->node_cnt
+		= node_cnt;
+
+	xfree(layout->node_list);
+	layout->node_list = xstrdup(node_list);
+	for (i = orig_task_num; i < layout->node_cnt; i++) {
+		layout->tasks[i] = 1;
+		layout->tids[i] = (uint32_t *)xmalloc(sizeof(uint32_t));
+		layout->tids[i][0] = (*curr_task_num)++;
+	}
+
+	/* Alter the launch state structure now that the settings
+	 * have changed */
+	if (!ctx->launch_state) {
+		ctx->launch_state = step_launch_state_create(ctx);
+		ctx->launch_state->slurmctld_socket_fd = sock;
+	} else
+		step_launch_state_alter(ctx);
 
 	return SLURM_SUCCESS;
 }
