@@ -90,7 +90,7 @@ extern char **environ;
  **********************************************************************/
 static int _launch_tasks(slurm_step_ctx_t *ctx,
 			 launch_tasks_request_msg_t *launch_msg,
-			 uint32_t timeout);
+			 uint32_t timeout, char *nodelist, int start_nodeid);
 static char *_lookup_cwd(void);
 static void _print_launch_msg(launch_tasks_request_msg_t *msg,
 			      char *hostname, int nodeid);
@@ -138,6 +138,7 @@ void slurm_step_launch_params_t_init (slurm_step_launch_params_t *ptr)
 	memcpy(&ptr->local_fds, &fds, sizeof(fds));
 	ptr->gid = getgid();
 	ptr->acctg_freq  = (uint16_t) NO_VAL;
+	ptr->cpu_freq = NO_VAL;
 }
 
 /*
@@ -250,6 +251,7 @@ int slurm_step_launch (slurm_step_ctx_t *ctx,
 	launch.task_epilog	= params->task_epilog;
 	launch.cpu_bind_type	= params->cpu_bind_type;
 	launch.cpu_bind		= params->cpu_bind;
+	launch.cpu_freq		= params->cpu_freq;
 	launch.mem_bind_type	= params->mem_bind_type;
 	launch.mem_bind		= params->mem_bind;
 	launch.multi_prog	= params->multi_prog ? 1 : 0;
@@ -327,17 +329,15 @@ int slurm_step_launch (slurm_step_ctx_t *ctx,
 		launch.resp_port[i] = ctx->launch_state->resp_port[i];
 	}
 
-	rc = _launch_tasks(ctx, &launch, params->msg_timeout);
+	rc = _launch_tasks(ctx, &launch, params->msg_timeout,
+			   launch.complete_nodelist, 0);
 
 	/* clean up */
 	xfree(launch.resp_port);
 	if (!ctx->launch_state->user_managed_io) {
 		xfree(launch.io_port);
 	}
-	goto done;
 fail1:
-
-done:
 	xfree(launch.complete_nodelist);
 	xfree(launch.cwd);
 	env_array_free(env);
@@ -345,6 +345,147 @@ done:
 	return rc;
 }
 
+/*
+ * slurm_step_launch_add - Add tasks to a step that was already started
+ * IN ctx - job step context generated by slurm_step_ctx_create
+ * IN params - job step parameters
+ * IN node_list - list of extra nodes to add
+ * IN start_nodeid - in the global scheme which node id is the first
+ *                   node in node_list.
+ * RET SLURM_SUCCESS or SLURM_ERROR (with errno set)
+ */
+int slurm_step_launch_add (slurm_step_ctx_t *ctx,
+			   const slurm_step_launch_params_t *params,
+			   char *node_list, int start_nodeid)
+{
+	launch_tasks_request_msg_t launch;
+	int i;
+	char **env = NULL;
+	char **mpi_env = NULL;
+	int rc = SLURM_SUCCESS;
+
+	debug("Entering slurm_step_launch_add");
+	if (!ctx->launch_state->user_managed_io)
+		fatal("slurm_step_launch_add has only been tested "
+		      "with user managed io");
+
+	memset(&launch, 0, sizeof(launch));
+
+	if (ctx == NULL || ctx->magic != STEP_CTX_MAGIC) {
+		error("Not a valid slurm_step_ctx_t!");
+		slurm_seterrno(EINVAL);
+		return SLURM_ERROR;
+	}
+
+	/* Now, hack the step_layout struct if the following it true.
+	   This looks like an ugly hack to support LAM/MPI's lamboot.
+	   NOTE: This also gets ran for BGQ systems. */
+	if (mpi_hook_client_single_task_per_node()) {
+		for (i = 0; i < ctx->step_resp->step_layout->node_cnt; i++)
+			ctx->step_resp->step_layout->tasks[i] = 1;
+	}
+
+	/* Start tasks on compute nodes */
+	launch.job_id = ctx->step_req->job_id;
+	launch.uid = ctx->step_req->user_id;
+	launch.gid = params->gid;
+	launch.argc = params->argc;
+	launch.argv = params->argv;
+	launch.spank_job_env = params->spank_job_env;
+	launch.spank_job_env_size = params->spank_job_env_size;
+	launch.cred = ctx->step_resp->cred;
+	launch.job_step_id = ctx->step_resp->job_step_id;
+	if (params->env == NULL) {
+		/* if the user didn't specify an environment, grab the
+		 * environment of the running process */
+		env_array_merge(&env, (const char **)environ);
+	} else {
+		env_array_merge(&env, (const char **)params->env);
+	}
+	env_array_for_step(&env, ctx->step_resp,
+			   ctx->launch_state->resp_port[0],
+			   params->preserve_env);
+	env_array_merge(&env, (const char **)mpi_env);
+	env_array_free(mpi_env);
+
+	launch.envc = envcount(env);
+	launch.env = env;
+	if (params->cwd != NULL) {
+		launch.cwd = xstrdup(params->cwd);
+	} else {
+		launch.cwd = _lookup_cwd();
+	}
+	launch.alias_list	= params->alias_list;
+	launch.nnodes		= ctx->step_resp->step_layout->node_cnt;
+	launch.ntasks		= ctx->step_resp->step_layout->task_cnt;
+	launch.slurmd_debug	= params->slurmd_debug;
+	launch.switch_job	= ctx->step_resp->switch_job;
+	launch.task_prolog	= params->task_prolog;
+	launch.task_epilog	= params->task_epilog;
+	launch.cpu_bind_type	= params->cpu_bind_type;
+	launch.cpu_bind		= params->cpu_bind;
+	launch.cpu_freq		= params->cpu_freq;
+	launch.mem_bind_type	= params->mem_bind_type;
+	launch.mem_bind		= params->mem_bind;
+	launch.multi_prog	= params->multi_prog ? 1 : 0;
+	launch.cpus_per_task	= params->cpus_per_task;
+	launch.task_dist	= params->task_dist;
+	launch.pty              = params->pty;
+	launch.ckpt_dir         = params->ckpt_dir;
+	launch.restart_dir      = params->restart_dir;
+	launch.acctg_freq	= params->acctg_freq;
+	launch.open_mode        = params->open_mode;
+	launch.options          = job_options_create();
+	launch.complete_nodelist =
+		xstrdup(ctx->step_resp->step_layout->node_list);
+
+	spank_set_remote_options (launch.options);
+	launch.task_flags = 0;
+	if (params->parallel_debug)
+		launch.task_flags |= TASK_PARALLEL_DEBUG;
+
+	launch.tasks_to_launch = ctx->step_resp->step_layout->tasks;
+	launch.cpus_allocated  = ctx->step_resp->step_layout->tasks;
+	launch.global_task_ids = ctx->step_resp->step_layout->tids;
+
+	launch.select_jobinfo  = ctx->step_resp->select_jobinfo;
+
+	launch.user_managed_io = params->user_managed_io ? 1 : 0;
+
+	/* user_managed_io is true */
+	if (!ctx->launch_state->io.user) {
+		/* initialize user_managed_io_t */
+		ctx->launch_state->io.user =
+			(user_managed_io_t *)xmalloc(sizeof(user_managed_io_t));
+		ctx->launch_state->io.user->connected = 0;
+		ctx->launch_state->io.user->sockets =
+			(int *)xmalloc(sizeof(int)*ctx->step_req->num_tasks);
+	} else
+		xrealloc(ctx->launch_state->io.user->sockets,
+			 sizeof(int)*ctx->step_req->num_tasks);
+
+	launch.num_resp_port = ctx->launch_state->num_resp_port;
+	launch.resp_port = xmalloc(sizeof(uint16_t) * launch.num_resp_port);
+	for (i = 0; i < launch.num_resp_port; i++) {
+		launch.resp_port[i] = ctx->launch_state->resp_port[i];
+	}
+
+	rc = _launch_tasks(ctx, &launch, params->msg_timeout,
+			   node_list, start_nodeid);
+
+	/* clean up */
+	xfree(launch.resp_port);
+	if (!ctx->launch_state->user_managed_io) {
+		xfree(launch.io_port);
+	}
+
+	xfree(launch.cwd);
+	env_array_free(env);
+	job_options_destroy(launch.options);
+
+	return rc;
+}
+
 static void _step_abort(slurm_step_ctx_t *ctx)
 {
 	struct step_launch_state *sls = ctx->launch_state;
@@ -378,7 +519,10 @@ int slurm_step_launch_wait_start(slurm_step_ctx_t *ctx)
 		}
 		if (pthread_cond_timedwait(&sls->cond, &sls->lock, &ts) ==
 		    ETIMEDOUT) {
-			error("timeout waiting for task launch");
+			error("timeout waiting for task launch, "
+			      "started %d of %d tasks",
+			      bit_set_count(sls->tasks_started),
+			      sls->tasks_requested);
 			sls->abort = true;
 			_step_abort(ctx);
 			pthread_cond_broadcast(&sls->cond);
@@ -506,8 +650,10 @@ void slurm_step_launch_wait_finish(slurm_step_ctx_t *ctx)
 	pthread_mutex_lock(&sls->lock);
 	pmi_kvs_free();
 
-	if (sls->msg_handle)
+	if (sls->msg_handle) {
 		eio_handle_destroy(sls->msg_handle);
+		sls->msg_handle = NULL;
+	}
 
 	/* Shutdown the io timeout thread, if one exists */
 	if (sls->io_timeout_thread_created) {
@@ -526,6 +672,7 @@ void slurm_step_launch_wait_finish(slurm_step_ctx_t *ctx)
 		pthread_mutex_lock(&sls->lock);
 
 		client_io_handler_destroy(sls->io.normal);
+		sls->io.normal = NULL;
 	}
 
 	mpi_hook_client_fini(sls->mpi_state);
@@ -663,8 +810,9 @@ struct step_launch_state *step_launch_state_create(slurm_step_ctx_t *ctx)
 	sls = xmalloc(sizeof(struct step_launch_state));
 	sls->slurmctld_socket_fd = -1;
 #if defined HAVE_BGQ
-//#if defined HAVE_BGQ && defined HAVE_BG_FILES
-	sls->tasks_requested = 1;
+	/* This means we are on an emulated system, so only launch 1
+	   task to avoid overflows with large jobs. */
+	layout->node_cnt = layout->task_cnt = sls->tasks_requested = 1;
 #else
 	/* Hack for LAM-MPI's lamboot, launch one task per node */
 	if (mpi_hook_client_single_task_per_node())
@@ -696,6 +844,38 @@ struct step_launch_state *step_launch_state_create(slurm_step_ctx_t *ctx)
 	return sls;
 }
 
+/*
+ * If a steps size has changed update the launch_state structure for a
+ * specified step context, "ctx".
+ */
+void step_launch_state_alter(slurm_step_ctx_t *ctx)
+{
+	struct step_launch_state *sls = ctx->launch_state;
+	slurm_step_layout_t *layout = ctx->step_resp->step_layout;
+	int ii;
+
+	xassert(sls);
+#if defined HAVE_BGQ
+//#if defined HAVE_BGQ && defined HAVE_BG_FILES
+	sls->tasks_requested = 1;
+#else
+	/* Hack for LAM-MPI's lamboot, launch one task per node */
+	if (mpi_hook_client_single_task_per_node())
+		sls->tasks_requested = layout->node_cnt;
+	else
+		sls->tasks_requested = layout->task_cnt;
+#endif
+	sls->tasks_started = bit_realloc(sls->tasks_started, layout->task_cnt);
+	sls->tasks_exited = bit_realloc(sls->tasks_exited, layout->task_cnt);
+	sls->node_io_error = bit_realloc(sls->node_io_error, layout->node_cnt);
+	xrealloc(sls->io_deadline, sizeof(time_t) * layout->node_cnt);
+	sls->layout = sls->mpi_info->step_layout = layout;
+
+	for (ii = 0; ii < layout->node_cnt; ii++) {
+		sls->io_deadline[ii] = (time_t)NO_VAL;
+	}
+}
+
 /*
  * Free the memory associated with the a launch state structure.
  */
@@ -946,9 +1126,12 @@ _job_complete_handler(struct step_launch_state *sls, slurm_msg_t *complete_msg)
 			step_msg->job_id, step_msg->step_id);
 	}
 
-	/* FIXME: does nothing yet */
+	if (sls->callback.step_complete)
+		(sls->callback.step_complete)(step_msg);
 
+	force_terminated_job = true;
 	pthread_mutex_lock(&sls->lock);
+	sls->abort = true;
 	pthread_cond_broadcast(&sls->cond);
 	pthread_mutex_unlock(&sls->lock);
 }
@@ -956,7 +1139,12 @@ _job_complete_handler(struct step_launch_state *sls, slurm_msg_t *complete_msg)
 static void
 _timeout_handler(struct step_launch_state *sls, slurm_msg_t *timeout_msg)
 {
-	/* FIXME: does nothing yet */
+	srun_timeout_msg_t *step_msg =
+		(srun_timeout_msg_t *) timeout_msg->data;
+
+	if (sls->callback.step_timeout)
+		(sls->callback.step_timeout)(step_msg);
+
 	pthread_mutex_lock(&sls->lock);
 	pthread_cond_broadcast(&sls->cond);
 	pthread_mutex_unlock(&sls->lock);
@@ -1175,14 +1363,18 @@ _step_missing_handler(struct step_launch_state *sls, slurm_msg_t *missing_msg)
 	hostset_destroy(all_nodes);
 }
 
-/* This RPC will probably only be used on BlueGene/Q systems to signal the
- * runjob process */
+/* This RPC typically used to send a signal an external program that
+ * is usually wrapped by srun.
+ */
 static void
 _step_step_signal(struct step_launch_state *sls, slurm_msg_t *signal_msg)
 {
 	job_step_kill_msg_t *step_signal = signal_msg->data;
 	debug2("Signal %u requested for step %u.%u", step_signal->signal,
 	       step_signal->job_id, step_signal->job_step_id);
+	if (sls->callback.step_signal)
+		(sls->callback.step_signal)(step_signal->signal);
+
 }
 
 /*
@@ -1265,11 +1457,6 @@ _handle_msg(void *arg, slurm_msg_t *msg)
 		break;
 	case SRUN_JOB_COMPLETE:
 		debug2("received job step complete message");
-		force_terminated_job = true;
-		pthread_mutex_lock(&sls->lock);
-		sls->abort = true;
-		pthread_cond_broadcast(&sls->cond);
-		pthread_mutex_unlock(&sls->lock);
 		_job_complete_handler(sls, msg);
 		slurm_free_srun_job_complete_msg(msg->data);
 		break;
@@ -1367,7 +1554,7 @@ static int _fail_step_tasks(slurm_step_ctx_t *ctx, char *node, int ret_code)
 
 static int _launch_tasks(slurm_step_ctx_t *ctx,
 			 launch_tasks_request_msg_t *launch_msg,
-			 uint32_t timeout)
+			 uint32_t timeout, char *nodelist, int start_nodeid)
 {
 #ifdef HAVE_FRONT_END
 	slurm_cred_arg_t cred_args;
@@ -1382,8 +1569,8 @@ static int _launch_tasks(slurm_step_ctx_t *ctx,
 	debug("Entering _launch_tasks");
 	if (ctx->verbose_level) {
 		char *name = NULL;
-		hostlist_t hl = hostlist_create(launch_msg->complete_nodelist);
-		int i = 0;
+		hostlist_t hl = hostlist_create(nodelist);
+		int i = start_nodeid;
 		while ((name = hostlist_shift(hl))) {
 			_print_launch_msg(launch_msg, name, i++);
 			free(name);
@@ -1402,7 +1589,7 @@ static int _launch_tasks(slurm_step_ctx_t *ctx,
 					false);
 	slurm_cred_free_args(&cred_args);
 #else
-	ret_list = slurm_send_recv_msgs(ctx->step_resp->step_layout->node_list,
+	ret_list = slurm_send_recv_msgs(nodelist,
 					&msg, timeout, false);
 #endif
 	if (ret_list == NULL) {
