@@ -119,13 +119,32 @@ struct job_resources {
 	uint32_t *	memory_used;
 	uint32_t	nhosts;
 	bitstr_t *	node_bitmap;
-	uint8_t		node_req;
+	uint32_t	node_req;
 	char *		nodes;
 	uint32_t	ncpus;
 	uint32_t *	sock_core_rep_count;
 	uint16_t *	sockets_per_node;
 };
 
+/*
+ * node_res_record.node_state assists with the unique state of each node.
+ * When a job is allocated, these flags provide protection for nodes in a
+ * Shared=NO or Shared=EXCLUSIVE partition from other jobs.
+ *
+ * NOTES:
+ * - If node is in use by Shared=NO part, some CPUs/memory may be available
+ * - Caution with NODE_CR_AVAILABLE: a Sharing partition could be full.
+ *
+ * - these values are staggered so that they can be incremented as multiple
+ *   jobs are allocated to each node. This is needed to be able to support
+ *   preemption, which can override these protections.
+ */
+enum node_cr_state {
+	NODE_CR_AVAILABLE = 0,    /* The node may be IDLE or IN USE (shared) */
+	NODE_CR_ONE_ROW = 1,      /* node is in use by Shared=NO part */
+	NODE_CR_RESERVED = 64000  /* node is in use by Shared=EXCLUSIVE part */
+};
+
 /* Create an empty job_resources data structure, just a call to xmalloc() */
 extern job_resources_t *create_job_resources(void);
 
