@@ -70,6 +70,7 @@
 #include "src/common/parse_time.h"
 #include "src/common/read_config.h"
 #include "src/common/slurm_accounting_storage.h"
+#include "src/common/slurm_acct_gather_energy.h"
 #include "src/common/slurm_topology.h"
 #include "src/common/xassert.h"
 #include "src/common/xmalloc.h"
@@ -86,6 +87,9 @@ struct node_record *node_record_table_ptr = NULL;	/* node records */
 struct node_record **node_hash_table = NULL;	/* node_record hash table */
 int node_record_count = 0;		/* count in node_record_table_ptr */
 
+uint16_t *cr_node_num_cores = NULL;
+uint32_t *cr_node_cores_offset = NULL;
+
 static void	_add_config_feature(char *feature, bitstr_t *node_bitmap);
 static int	_build_single_nodeline_info(slurm_conf_node_t *node_ptr,
 					    struct config_record *config_ptr);
@@ -143,14 +147,17 @@ static int _build_single_nodeline_info(slurm_conf_node_t *node_ptr,
 {
 	int error_code = SLURM_SUCCESS;
 	struct node_record *node_rec = NULL;
+	hostlist_t address_list = NULL;
 	hostlist_t alias_list = NULL;
 	hostlist_t hostname_list = NULL;
-	hostlist_t address_list = NULL;
+	hostlist_t port_list = NULL;
+	char *address = NULL;
 	char *alias = NULL;
 	char *hostname = NULL;
-	char *address = NULL;
+	char *port_str = NULL;
 	int state_val = NODE_STATE_UNKNOWN;
-	int address_count, alias_count, hostname_count;
+	int address_count, alias_count, hostname_count, port_count;
+	uint16_t port = 0;
 
 	if (node_ptr->state != NULL) {
 		state_val = state_str2int(node_ptr->state, node_ptr->nodenames);
@@ -158,6 +165,12 @@ static int _build_single_nodeline_info(slurm_conf_node_t *node_ptr,
 			goto cleanup;
 	}
 
+	if ((address_list = hostlist_create(node_ptr->addresses)) == NULL) {
+		fatal("Unable to create NodeAddr list from %s",
+		      node_ptr->addresses);
+		error_code = errno;
+		goto cleanup;
+	}
 	if ((alias_list = hostlist_create(node_ptr->nodenames)) == NULL) {
 		fatal("Unable to create NodeName list from %s",
 		      node_ptr->nodenames);
@@ -170,9 +183,19 @@ static int _build_single_nodeline_info(slurm_conf_node_t *node_ptr,
 		error_code = errno;
 		goto cleanup;
 	}
-	if ((address_list = hostlist_create(node_ptr->addresses)) == NULL) {
-		fatal("Unable to create NodeAddr list from %s",
-		      node_ptr->addresses);
+	if (node_ptr->port_str && node_ptr->port_str[0] &&
+	    (node_ptr->port_str[0] != '[') &&
+	    (strchr(node_ptr->port_str, '-') ||
+	     strchr(node_ptr->port_str, ','))) {
+		xstrfmtcat(port_str, "[%s]", node_ptr->port_str);
+		port_list = hostlist_create(port_str);
+		xfree(port_str);
+	} else {
+		port_list = hostlist_create(node_ptr->port_str);
+	}
+	if (port_list == NULL) {
+		error("Unable to create Port list from %s",
+		      node_ptr->port_str);
 		error_code = errno;
 		goto cleanup;
 	}
@@ -181,6 +204,7 @@ static int _build_single_nodeline_info(slurm_conf_node_t *node_ptr,
 	address_count  = hostlist_count(address_list);
 	alias_count    = hostlist_count(alias_list);
 	hostname_count = hostlist_count(hostname_list);
+	port_count     = hostlist_count(port_list);
 #ifdef HAVE_FRONT_END
 	if ((hostname_count != alias_count) && (hostname_count != 1)) {
 		error("NodeHostname count must equal that of NodeName "
@@ -193,30 +217,54 @@ static int _build_single_nodeline_info(slurm_conf_node_t *node_ptr,
 		goto cleanup;
 	}
 #else
+#ifdef MULTIPLE_SLURMD
+	if ((address_count != alias_count) && (address_count != 1)) {
+		error("NodeAddr count must equal that of NodeName "
+		      "records of there must be no more than one");
+		goto cleanup;
+	}
+#else
+	if (address_count < alias_count) {
+		error("At least as many NodeAddr are required as NodeName");
+		goto cleanup;
+	}
 	if (hostname_count < alias_count) {
 		error("At least as many NodeHostname are required "
 		      "as NodeName");
 		goto cleanup;
 	}
-	if (address_count < alias_count) {
-		error("At least as many NodeAddr are required as NodeName");
+#endif	/* MULTIPLE_SLURMD */
+#endif	/* HAVE_FRONT_END */
+	if ((port_count != alias_count) && (port_count > 1)) {
+		error("Port count must equal that of NodeName "
+		      "records or there must be no more than one");
 		goto cleanup;
 	}
-#endif
 
 	/* now build the individual node structures */
 	while ((alias = hostlist_shift(alias_list))) {
+		if (address_count > 0) {
+			address_count--;
+			if (address)
+				free(address);
+			address = hostlist_shift(address_list);
+		}
 		if (hostname_count > 0) {
 			hostname_count--;
 			if (hostname)
 				free(hostname);
 			hostname = hostlist_shift(hostname_list);
 		}
-		if (address_count > 0) {
-			address_count--;
-			if (address)
-				free(address);
-			address = hostlist_shift(address_list);
+		if (port_count > 0) {
+			int port_int;
+			port_count--;
+			if (port_str)
+				free(port_str);
+			port_str = hostlist_shift(port_list);
+			port_int = atoi(port_str);
+			if ((port_int <= 0) || (port_int > 0xffff))
+				fatal("Invalid Port %s", node_ptr->port_str);
+			port = port_int;
 		}
 		/* find_node_record locks this to get the
 		 * alias so we need to unlock */
@@ -230,7 +278,7 @@ static int _build_single_nodeline_info(slurm_conf_node_t *node_ptr,
 			node_rec->last_response = (time_t) 0;
 			node_rec->comm_name = xstrdup(address);
 			node_rec->node_hostname = xstrdup(hostname);
-			node_rec->port      = node_ptr->port;
+			node_rec->port      = port;
 			node_rec->weight    = node_ptr->weight;
 			node_rec->features  = xstrdup(node_ptr->feature);
 			node_rec->reason    = xstrdup(node_ptr->reason);
@@ -247,12 +295,16 @@ cleanup:
 		free(address);
 	if (hostname)
 		free(hostname);
+	if (port_str)
+		free(port_str);
+	if (address_list)
+		hostlist_destroy(address_list);
 	if (alias_list)
 		hostlist_destroy(alias_list);
 	if (hostname_list)
 		hostlist_destroy(hostname_list);
-	if (address_list)
-		hostlist_destroy(address_list);
+	if (port_list)
+		hostlist_destroy(port_list);
 	return error_code;
 }
 
@@ -596,6 +648,7 @@ extern int build_all_nodeline_info (bool set_bitmap)
 		config_ptr = create_config_record();
 		config_ptr->nodes = xstrdup(node->nodenames);
 		config_ptr->cpus = node->cpus;
+		config_ptr->boards = node->boards;
 		config_ptr->sockets = node->sockets;
 		config_ptr->cores = node->cores;
 		config_ptr->threads = node->threads;
@@ -730,12 +783,15 @@ extern struct node_record *create_node_record (
 	node_ptr->config_ptr = config_ptr;
 	/* these values will be overwritten when the node actually registers */
 	node_ptr->cpus = config_ptr->cpus;
+	node_ptr->cpu_load = NO_VAL;
+	node_ptr->boards = config_ptr->boards;
 	node_ptr->sockets = config_ptr->sockets;
 	node_ptr->cores = config_ptr->cores;
 	node_ptr->threads = config_ptr->threads;
 	node_ptr->real_memory = config_ptr->real_memory;
 	node_ptr->tmp_disk = config_ptr->tmp_disk;
 	node_ptr->select_nodeinfo = select_g_select_nodeinfo_alloc();
+	node_ptr->energy = acct_gather_energy_alloc();
 	xassert (node_ptr->magic = NODE_MAGIC)  /* set value */;
 	return node_ptr;
 }
@@ -920,6 +976,7 @@ extern void purge_node_rec (struct node_record *node_ptr)
 	xfree(node_ptr->os);
 	xfree(node_ptr->part_pptr);
 	xfree(node_ptr->reason);
+	acct_gather_energy_destroy(node_ptr->energy);
 	select_g_select_nodeinfo_free(node_ptr->select_nodeinfo);
 }
 
@@ -981,3 +1038,53 @@ extern int state_str2int(const char *state_str, char *node_name)
 	}
 	return state_val;
 }
+
+/* (re)set cr_node_num_cores arrays */
+extern void cr_init_global_core_data(struct node_record *node_ptr, int node_cnt,
+				     uint16_t fast_schedule)
+{
+	uint32_t n;
+
+	cr_fini_global_core_data();
+
+	cr_node_num_cores = xmalloc(node_cnt * sizeof(uint16_t));
+	cr_node_cores_offset = xmalloc((node_cnt+1) * sizeof(uint32_t));
+
+	for (n = 0; n < node_cnt; n++) {
+		uint16_t cores;
+		if (fast_schedule) {
+			cores  = node_ptr[n].config_ptr->cores;
+			cores *= node_ptr[n].config_ptr->sockets;
+		} else {
+			cores  = node_ptr[n].cores;
+			cores *= node_ptr[n].sockets;
+		}
+		cr_node_num_cores[n] = cores;
+		if (n > 0) {
+			cr_node_cores_offset[n] = cr_node_cores_offset[n-1] +
+						  cr_node_num_cores[n-1] ;
+		} else
+			cr_node_cores_offset[0] = 0;
+	}
+
+	/* an extra value is added to get the total number of cores */
+	/* as cr_get_coremap_offset is sometimes used to get the total */
+	/* number of cores in the cluster */
+	cr_node_cores_offset[node_cnt] = cr_node_cores_offset[node_cnt-1] +
+					 cr_node_num_cores[node_cnt-1] ;
+
+}
+
+extern void cr_fini_global_core_data(void)
+{
+	xfree(cr_node_num_cores);
+	xfree(cr_node_cores_offset);
+}
+
+/* return the coremap index to the first core of the given node */
+
+extern uint32_t cr_get_coremap_offset(uint32_t node_index)
+{
+	xassert(cr_node_cores_offset);
+	return cr_node_cores_offset[node_index];
+}
