@@ -889,7 +889,7 @@ int sig_name2num(char *signal_name)
 /*
  * parse_uint32 - Convert anscii string to a 32 bit unsigned int.
  * IN      aval - ascii string.
- * IN/OUT  ival - 32 bit pointer. 
+ * IN/OUT  ival - 32 bit pointer.
  * RET     0 if no error, 1 otherwise.
  */
 extern int parse_uint32(char *aval, uint32_t *ival)
@@ -921,7 +921,7 @@ extern int parse_uint32(char *aval, uint32_t *ival)
 /*
  * parse_uint16 - Convert anscii string to a 16 bit unsigned int.
  * IN      aval - ascii string.
- * IN/OUT  ival - 16 bit pointer. 
+ * IN/OUT  ival - 16 bit pointer.
  * RET     0 if no error, 1 otherwise.
  */
 extern int parse_uint16(char *aval, uint16_t *ival)
@@ -949,3 +949,228 @@ extern int parse_uint16(char *aval, uint16_t *ival)
 
 	return 0;
 }
+
+/* print_db_notok() - Print an error message about slurmdbd
+ *                    is unreachable or wrong cluster name.
+ * IN  cname - char * cluster name
+ * IN  isenv - bool  cluster name from env or from command line option.
+ */
+void print_db_notok(const char *cname, bool isenv)
+{
+	if (errno)
+		error("There is a problem talking to the database: %m.  "
+		      "Only local cluster communication is available, remove "
+		      "%s or contact your admin to resolve the problem.",
+		      isenv ? "SLURM_CLUSTERS from your environment" :
+		      "--cluster from your command line");
+	else if (!strcasecmp("all", cname))
+		error("No clusters can be reached now. "
+		      "Contact your admin to resolve the problem.");
+	else
+		error("'%s' can't be reached now, "
+		      "or it is an invalid entry for %s.  "
+		      "Use 'sacctmgr list clusters' to see available clusters.",
+		      cname, isenv ? "SLURM_CLUSTERS" : "--cluster");
+}
+
+static bool _check_is_pow_of_2(int32_t n) {
+	/* Bitwise ANDing a power of 2 number like 16 with its
+	 * negative (-16) gives itself back.  Only integers which are power of
+	 * 2 behave like that.
+	 */
+	return ((n!=0) && (n&(-n))==n);
+}
+
+extern void bg_figure_nodes_tasks(int *min_nodes, int *max_nodes,
+				  int *ntasks_per_node, bool *ntasks_set,
+				  int *ntasks, bool nodes_set,
+				  bool nodes_set_opt, bool overcommit,
+				  bool set_tasks)
+{
+	/* BGQ has certain restrictions to run a job.  So lets validate
+	 * and correct what the user asked for if possible.
+	 */
+	int32_t node_cnt;
+	bool figured = false;
+	uint32_t cluster_flags = slurmdb_setup_cluster_flags();
+
+	if (!(cluster_flags & CLUSTER_FLAG_BGQ))
+		fatal("bg_figure_nodes_tasks is only valid on a BGQ system.");
+
+	if (!(*ntasks_set)
+	    && (*ntasks_per_node) && (*ntasks_per_node != NO_VAL)) {
+		if ((*ntasks_per_node != 1)
+		    && (*ntasks_per_node != 2)
+		    && (*ntasks_per_node != 4)
+		    && (*ntasks_per_node != 8)
+		    && (*ntasks_per_node != 16)
+		    && (*ntasks_per_node != 32)
+		    && (*ntasks_per_node != 64))
+			fatal("You requested --ntasks-per-node=%d, "
+			      "which is not valid, it must be a power of 2.  "
+			      "Please validate your request and try again.",
+			      *ntasks_per_node);
+		else if (!overcommit
+			 && ((*ntasks_per_node == 32)
+			     || (*ntasks_per_node == 64)))
+			fatal("You requested --ntasks-per-node=%d, "
+			      "which is not valid without --overcommit.",
+			      *ntasks_per_node);
+	}
+
+	if (*max_nodes)
+		node_cnt = *max_nodes;
+	else
+		node_cnt = *min_nodes;
+
+	if (*ntasks_set) {
+		int32_t ntpn;
+
+		if (nodes_set) {
+			if (node_cnt > *ntasks) {
+				if (nodes_set_opt)
+					info("You asked for %d nodes, "
+					     "but only %d tasks, resetting "
+					     "node count to %u.",
+					     node_cnt, *ntasks, *ntasks);
+				*max_nodes = *min_nodes = node_cnt
+					= *ntasks;
+			}
+		}
+		/* If nodes not set do not try to set min/max nodes
+		   yet since that would result in an incorrect
+		   allocation.  For a step allocation it is figured
+		   out later in srun_job.c _job_create_structure().
+		*/
+
+		if ((!*ntasks_per_node || (*ntasks_per_node == NO_VAL))) {
+			/* We always want the next larger number if
+			   there is a fraction so we try to stay in
+			   the allocation requested.
+			*/
+			*ntasks_per_node =
+				(*ntasks + node_cnt - 1) / node_cnt;
+			figured = true;
+		}
+
+		/* On a Q we need ntasks_per_node to be a multiple of 2 */
+		ntpn = *ntasks_per_node;
+		while (!_check_is_pow_of_2(ntpn))
+			ntpn++;
+		if (!figured && ntpn > 64)
+			fatal("You requested --ntasks-per-node=%d, "
+			      "which is not a power of 2.  But the next "
+			      "largest power of 2 (%d) is greater than the "
+			      "largest valid power which is 64.  Please "
+			      "validate your request and try again.",
+			      *ntasks_per_node, ntpn);
+		if (!figured && (ntpn != *ntasks_per_node)) {
+			info("You requested --ntasks-per-node=%d, which is not "
+			     "a power of 2.  Setting --ntasks-per-node=%d "
+			     "for you.", *ntasks_per_node, ntpn);
+			figured = true;
+		}
+		*ntasks_per_node = ntpn;
+
+		/* We always want the next larger number if
+		   there is a fraction so we try to stay in
+		   the allocation requested.
+		*/
+		ntpn = ((*ntasks) + (*ntasks_per_node) - 1)
+			/ (*ntasks_per_node);
+		/* Make sure we are requesting the correct number of nodes. */
+		if (node_cnt < ntpn) {
+			*max_nodes = *min_nodes = ntpn;
+			if (nodes_set && !figured) {
+				fatal("You requested -N %d and -n %d "
+				      "with --ntasks-per-node=%d.  "
+				      "This isn't a valid request.",
+				      node_cnt, *ntasks,
+				      *ntasks_per_node);
+			}
+			node_cnt = *max_nodes;
+		}
+
+		/* Do this again to make sure we have a legitimate
+		   ratio. */
+		ntpn = *ntasks_per_node;
+		if ((node_cnt * ntpn) < *ntasks) {
+			ntpn++;
+			while (!_check_is_pow_of_2(ntpn))
+				ntpn++;
+			if (!figured && (ntpn != *ntasks_per_node))
+				info("You requested --ntasks-per-node=%d, "
+				     "which cannot spread across %d nodes "
+				     "correctly.  Setting --ntasks-per-node=%d "
+				     "for you.",
+				     *ntasks_per_node, node_cnt, ntpn);
+			*ntasks_per_node = ntpn;
+		} else if ((node_cnt * ntpn) > *ntasks) {
+			ntpn = (*ntasks + node_cnt - 1) / node_cnt;
+			while (!_check_is_pow_of_2(ntpn))
+				ntpn++;
+			if (!figured && (ntpn != *ntasks_per_node))
+				info("You requested --ntasks-per-node=%d, "
+				     "which is more than the tasks you "
+				     "requested.  Setting --ntasks-per-node=%d "
+				     "for you.",
+				     *ntasks_per_node, ntpn);
+			*ntasks_per_node = ntpn;
+		}
+	} else if (set_tasks) {
+		if (*ntasks_per_node && (*ntasks_per_node != NO_VAL))
+			*ntasks = node_cnt * (*ntasks_per_node);
+		else {
+			*ntasks = node_cnt;
+			*ntasks_per_node = 1;
+		}
+		*ntasks_set = true;
+	}
+
+	/* If set_tasks isn't set we are coming in for the
+	   allocation so verify it will work first before we
+	   go any futher.
+	*/
+	if (nodes_set && (*ntasks_per_node && (*ntasks_per_node != NO_VAL))) {
+		if ((*ntasks_per_node != 1)
+		    && (*ntasks_per_node != 2)
+		    && (*ntasks_per_node != 4)
+		    && (*ntasks_per_node != 8)
+		    && (*ntasks_per_node != 16)
+		    && (*ntasks_per_node != 32)
+		    && (*ntasks_per_node != 64)) {
+			if (*ntasks_set)
+				fatal("You requested -N %d and -n %d "
+				      "which gives --ntasks-per-node=%d.  "
+				      "This isn't a valid request.",
+				      node_cnt, *ntasks,
+				      *ntasks_per_node);
+			else
+				fatal("You requested -N %d and "
+				      "--ntasks-per-node=%d.  "
+				      "This isn't a valid request.",
+				      node_cnt, *ntasks_per_node);
+		} else if (!overcommit
+			 && ((*ntasks_per_node == 32)
+			     || (*ntasks_per_node == 64))) {
+			if (*ntasks_set)
+				fatal("You requested -N %d and -n %d "
+				      "which gives --ntasks-per-node=%d.  "
+				      "This isn't a valid request "
+				      "without --overcommit.",
+				      node_cnt, *ntasks,
+				      *ntasks_per_node);
+			else
+				fatal("You requested -N %d and "
+				      "--ntasks-per-node=%d.  "
+				      "This isn't a valid request "
+				      "without --overcommit.",
+				      node_cnt, *ntasks_per_node);
+		}
+	}
+
+	/* If we aren't setting tasks reset ntasks_per_node as well. */
+	if (!set_tasks && figured)
+		*ntasks_per_node = 0;
+
+}
