@@ -72,7 +72,7 @@ void print_gres_help(void);
 task_dist_states_t verify_dist_type(const char *arg, uint32_t *plane_size);
 
 /*
- * verify comma separated list of connection types to array of uint16_t 
+ * verify comma separated list of connection types to array of uint16_t
  * connection_types or NO_VAL if not recognized
  */
 extern void verify_conn_type(const char *arg, uint16_t *conn_type);
@@ -148,7 +148,7 @@ int sig_name2num(char *signal_name);
 /*
  * parse_uint16 - Convert anscii string to a 16 bit unsigned int.
  * IN      aval - ascii string.
- * IN/OUT  ival - 16 bit pointer. 
+ * IN/OUT  ival - 16 bit pointer.
  * RET     0 if no error, 1 otherwise.
  */
 extern int	parse_uint16(char *aval, uint16_t *ival);
@@ -156,9 +156,24 @@ extern int	parse_uint16(char *aval, uint16_t *ival);
 /*
  * parse_uint32 - Convert anscii string to a 32 bit unsigned int.
  * IN      aval - ascii string.
- * IN/OUT  ival - 32 bit pointer. 
+ * IN/OUT  ival - 32 bit pointer.
  * RET     0 if no error, 1 otherwise.
  */
 extern int	parse_uint32(char *aval, uint32_t *ival);
 
+/* print_db_notok() - Print an error message about slurmdbd
+ *                    is unreachable or wrong cluster name.
+ * IN  cname - char * cluster name
+ * IN  isenv - bool   cluster name from env or from command line option.
+ */
+extern void print_db_notok(const char *cname, bool isenv);
+
+
+
+extern void bg_figure_nodes_tasks(int *min_nodes, int *max_nodes,
+				  int *ntasks_per_node, bool *ntasks_set,
+				  int *ntasks, bool nodes_set,
+				  bool nodes_set_opt, bool overcommit,
+				  bool set_tasks);
+
 #endif /* !_PROC_ARGS_H */
