@@ -129,7 +129,7 @@ void slurm_cred_ctx_destroy(slurm_cred_ctx_t ctx);
  *
  * On pack() ctx is packed in machine-independent format into the
  * buffer, on unpack() the contents of the buffer are used to
- * intialize the state of the context ctx.
+ * initialize the state of the context ctx.
  */
 int  slurm_cred_ctx_pack(slurm_cred_ctx_t ctx, Buf buffer);
 int  slurm_cred_ctx_unpack(slurm_cred_ctx_t ctx, Buf buffer);
@@ -337,7 +337,8 @@ void slurm_cred_print(slurm_cred_t *cred);
  * Caller of extract_sbcast_cred() must xfree returned node string
  */
 sbcast_cred_t *create_sbcast_cred(slurm_cred_ctx_t ctx,
-				 uint32_t job_id, char *nodes);
+				  uint32_t job_id, char *nodes,
+				  time_t expiration);
 sbcast_cred_t *copy_sbcast_cred(sbcast_cred_t *sbcast_cred);
 void          delete_sbcast_cred(sbcast_cred_t *sbcast_cred);
 int           extract_sbcast_cred(slurm_cred_ctx_t ctx,
