@@ -1,3 +1,4 @@
+
 /*****************************************************************************\
  *  slurm_jobcomp.c - implementation-independent job completion logging
  *  functions
@@ -70,147 +71,23 @@ typedef struct slurm_jobcomp_ops {
 	int          (*archive)   ( slurmdb_archive_cond_t *params );
 } slurm_jobcomp_ops_t;
 
-
 /*
- * A global job completion context.  "Global" in the sense that there's
- * only one, with static bindings.  We don't export it.
+ * These strings must be kept in the same order as the fields
+ * declared for slurm_jobcomp_ops_t.
  */
-
-struct slurm_jobcomp_context {
-	char *			jobcomp_type;
-	plugrack_t		plugin_list;
-	plugin_handle_t		cur_plugin;
-	int			jobcomp_errno;
-	slurm_jobcomp_ops_t	ops;
+static const char *syms[] = {
+	"slurm_jobcomp_set_location",
+	"slurm_jobcomp_log_record",
+	"slurm_jobcomp_get_errno",
+	"slurm_jobcomp_strerror",
+	"slurm_jobcomp_get_jobs",
+	"slurm_jobcomp_archive"
 };
 
-static slurm_jobcomp_context_t g_context = NULL;
-static pthread_mutex_t      context_lock = PTHREAD_MUTEX_INITIALIZER;
-
-static slurm_jobcomp_context_t
-_slurm_jobcomp_context_create( const char *jobcomp_type)
-{
-	slurm_jobcomp_context_t c;
-
-	if ( jobcomp_type == NULL ) {
-		debug3( "_slurm_jobcomp_context_create: no jobcomp type" );
-		return NULL;
-	}
-
-	c = xmalloc( sizeof( struct slurm_jobcomp_context ) );
-
-	c->jobcomp_errno = SLURM_SUCCESS;
-
-	/* Copy the job completion job completion type. */
-	c->jobcomp_type = xstrdup( jobcomp_type );
-	if ( c->jobcomp_type == NULL ) {
-		debug3( "can't make local copy of jobcomp type" );
-		xfree( c );
-		return NULL;
-	}
-
-	/* Plugin rack is demand-loaded on first reference. */
-	c->plugin_list = NULL;
-	c->cur_plugin = PLUGIN_INVALID_HANDLE;
-
-	return c;
-}
-
-static int
-_slurm_jobcomp_context_destroy( slurm_jobcomp_context_t c )
-{
-	int rc = SLURM_SUCCESS;
-	/*
-	 * Must check return code here because plugins might still
-	 * be loaded and active.
-	 */
-	if ( c->plugin_list ) {
-		if ( plugrack_destroy( c->plugin_list ) != SLURM_SUCCESS ) {
-			rc = SLURM_ERROR;
-		}
-	} else {
-		plugin_unload(c->cur_plugin);
-	}
-
-	xfree( c->jobcomp_type );
-	xfree( c );
-
-	return rc;
-}
-
-/*
- * Resolve the operations from the plugin.
- */
-static slurm_jobcomp_ops_t *
-_slurm_jobcomp_get_ops( slurm_jobcomp_context_t c )
-{
-	/*
-         * These strings must be kept in the same order as the fields
-         * declared for slurm_jobcomp_ops_t.
-         */
-	static const char *syms[] = {
-		"slurm_jobcomp_set_location",
-		"slurm_jobcomp_log_record",
-		"slurm_jobcomp_get_errno",
-		"slurm_jobcomp_strerror",
-		"slurm_jobcomp_get_jobs",
-		"slurm_jobcomp_archive"
-	};
-        int n_syms = sizeof( syms ) / sizeof( char * );
-
-	/* Find the correct plugin. */
-        c->cur_plugin = plugin_load_and_link(c->jobcomp_type, n_syms, syms,
-					     (void **) &c->ops);
-        if ( c->cur_plugin != PLUGIN_INVALID_HANDLE )
-        	return &c->ops;
-
-	if(errno != EPLUGIN_NOTFOUND) {
-		error("Couldn't load specified plugin name for %s: %s",
-		      c->jobcomp_type, plugin_strerror(errno));
-		return NULL;
-	}
-
-	error("Couldn't find the specified plugin name for %s "
-	      "looking at all files",
-	      c->jobcomp_type);
-
-	/* Get the plugin list, if needed. */
-        if ( c->plugin_list == NULL ) {
-		char *plugin_dir;
-                c->plugin_list = plugrack_create();
-                if ( c->plugin_list == NULL ) {
-                        error( "Unable to create a plugin manager" );
-                        return NULL;
-                }
-
-                plugrack_set_major_type( c->plugin_list, "jobcomp" );
-                plugrack_set_paranoia( c->plugin_list,
-				       PLUGRACK_PARANOIA_NONE,
-				       0 );
-		plugin_dir = slurm_get_plugin_dir();
-                plugrack_read_dir( c->plugin_list, plugin_dir );
-		xfree(plugin_dir);
-        }
-
-        /* Find the correct plugin. */
-        c->cur_plugin =
-		plugrack_use_by_type( c->plugin_list, c->jobcomp_type );
-        if ( c->cur_plugin == PLUGIN_INVALID_HANDLE ) {
-                error( "can't find a plugin for type %s", c->jobcomp_type );
-                return NULL;
-        }
-
-        /* Dereference the API. */
-        if ( plugin_get_syms( c->cur_plugin,
-                              n_syms,
-                              syms,
-                              (void **) &c->ops ) < n_syms ) {
-                error( "incomplete jobcomp plugin detected" );
-                return NULL;
-        }
-
-        return &c->ops;
-}
+static slurm_jobcomp_ops_t ops;
+static plugin_context_t *g_context = NULL;
+static pthread_mutex_t context_lock = PTHREAD_MUTEX_INITIALIZER;
+static bool init_run = false;
 
 extern void
 jobcomp_destroy_job(void *object)
@@ -241,33 +118,32 @@ extern int
 g_slurm_jobcomp_init( char *jobcomp_loc )
 {
 	int retval = SLURM_SUCCESS;
-	char *jobcomp_type;
+	char *plugin_type = "jobcomp";
+	char *type;
+
+	if (init_run && g_context)
+		return retval;
 
 	slurm_mutex_lock( &context_lock );
 
-	if ( g_context )
-		_slurm_jobcomp_context_destroy(g_context);
+	if (g_context)
+		plugin_context_destroy(g_context);
 
-	jobcomp_type = slurm_get_jobcomp_type();
-	g_context = _slurm_jobcomp_context_create( jobcomp_type );
-	if ( g_context == NULL ) {
-		error( "cannot create a context for %s", jobcomp_type );
-		xfree(jobcomp_type);
+	type = slurm_get_jobcomp_type();
+	g_context = plugin_context_create(
+		plugin_type, type, (void **)&ops, syms, sizeof(syms));
+
+	if (!g_context) {
+		error("cannot create %s context for %s", plugin_type, type);
 		retval = SLURM_ERROR;
 		goto done;
 	}
-	xfree(jobcomp_type);
-
-	if ( _slurm_jobcomp_get_ops( g_context ) == NULL ) {
-		error( "cannot resolve job completion plugin operations" );
-		_slurm_jobcomp_context_destroy( g_context );
-		g_context = NULL;
-		retval = SLURM_ERROR;
-	}
+	init_run = true;
 
 done:
-	if ( g_context )
-		retval = (*(g_context->ops.set_loc))(jobcomp_loc);
+	xfree(type);
+	if (g_context)
+		retval = (*(ops.set_loc))(jobcomp_loc);
 	slurm_mutex_unlock( &context_lock );
 	return retval;
 }
@@ -280,7 +156,8 @@ g_slurm_jobcomp_fini(void)
 	if ( !g_context)
 		goto done;
 
-	_slurm_jobcomp_context_destroy ( g_context );
+	init_run = false;
+	plugin_context_destroy ( g_context );
 	g_context = NULL;
 
 done:
@@ -295,7 +172,7 @@ g_slurm_jobcomp_write(struct job_record *job_ptr)
 
 	slurm_mutex_lock( &context_lock );
 	if ( g_context )
-		retval = (*(g_context->ops.job_write))(job_ptr);
+		retval = (*(ops.job_write))(job_ptr);
 	else {
 		error ("slurm_jobcomp plugin context not initialized");
 		retval = ENOENT;
@@ -311,7 +188,7 @@ g_slurm_jobcomp_errno(void)
 
 	slurm_mutex_lock( &context_lock );
 	if ( g_context )
-		retval = (*(g_context->ops.sa_errno))();
+		retval = (*(ops.sa_errno))();
 	else {
 		error ("slurm_jobcomp plugin context not initialized");
 		retval = ENOENT;
@@ -327,7 +204,7 @@ g_slurm_jobcomp_strerror(int errnum)
 
 	slurm_mutex_lock( &context_lock );
 	if ( g_context )
-		retval = (*(g_context->ops.job_strerror))(errnum);
+		retval = (*(ops.job_strerror))(errnum);
 	else
 		error ("slurm_jobcomp plugin context not initialized");
 	slurm_mutex_unlock( &context_lock );
@@ -341,7 +218,7 @@ g_slurm_jobcomp_get_jobs(slurmdb_job_cond_t *job_cond)
 
 	slurm_mutex_lock( &context_lock );
 	if ( g_context )
-		job_list = (*(g_context->ops.get_jobs))(job_cond);
+		job_list = (*(ops.get_jobs))(job_cond);
 	else
 		error ("slurm_jobcomp plugin context not initialized");
 	slurm_mutex_unlock( &context_lock );
@@ -355,7 +232,7 @@ g_slurm_jobcomp_archive(slurmdb_archive_cond_t *arch_cond)
 
 	slurm_mutex_lock( &context_lock );
 	if ( g_context )
-		rc = (*(g_context->ops.archive))(arch_cond);
+		rc = (*(ops.archive))(arch_cond);
 	else
 		error ("slurm_jobcomp plugin context not initialized");
 	slurm_mutex_unlock( &context_lock );
