@@ -99,11 +99,11 @@ slurm_step_layout_t *slurm_step_layout_create(
 	uint32_t cluster_flags = slurmdb_setup_cluster_flags();
 
 	step_layout->task_dist = task_dist;
-	if(task_dist == SLURM_DIST_ARBITRARY) {
+	if (task_dist == SLURM_DIST_ARBITRARY) {
 		hostlist_t hl = NULL;
 		char *buf = NULL;
 		/* set the node list for the task layout later if user
-		   supplied could be different that the job allocation */
+		 * supplied could be different that the job allocation */
 		arbitrary_nodes = xstrdup(tlist);
 		hl = hostlist_create(tlist);
 		hostlist_uniq(hl);
@@ -116,24 +116,24 @@ slurm_step_layout_t *slurm_step_layout_create(
 	}
 
 	step_layout->task_cnt  = num_tasks;
-	if(cluster_flags & CLUSTER_FLAG_FE) {
-		/* Limited job step support */
-		/* All jobs execute through front-end on Blue Gene.
+	if (cluster_flags & CLUSTER_FLAG_FE) {
+		/* Limited job step support on front-end systems.
+		 * All jobs execute through front-end on Blue Gene.
 		 * Normally we would not permit execution of job steps,
 		 * but can fake it by just allocating all tasks to
 		 * one of the allocated nodes. */
-		if(cluster_flags & CLUSTER_FLAG_BG)
+		if ((cluster_flags & CLUSTER_FLAG_BG)
+		    || (cluster_flags & CLUSTER_FLAG_CRAYXT))
 			step_layout->node_cnt  = num_hosts;
 		else
 			step_layout->node_cnt  = 1;
 	} else
 		step_layout->node_cnt  = num_hosts;
 
-	if(_init_task_layout(step_layout, arbitrary_nodes,
-			     cpus_per_node, cpu_count_reps,
-			     cpus_per_task,
-			     task_dist, plane_size)
-	   != SLURM_SUCCESS) {
+	if (_init_task_layout(step_layout, arbitrary_nodes,
+			      cpus_per_node, cpu_count_reps,
+			      cpus_per_task,
+			      task_dist, plane_size) != SLURM_SUCCESS) {
 		slurm_step_layout_destroy(step_layout);
 		step_layout = NULL;
 	}
@@ -164,38 +164,29 @@ slurm_step_layout_t *fake_slurm_step_layout_create(
 {
 	uint32_t cpn = 1;
 	int cpu_cnt = 0, cpu_inx = 0, i, j;
-/* 	char *name = NULL; */
-	hostlist_t hl = NULL;
 	slurm_step_layout_t *step_layout = NULL;
 
-	if((node_cnt <= 0) || (task_cnt <= 0 && !cpus_per_node) || !tlist) {
+	if ((node_cnt <= 0) || (task_cnt <= 0 && !cpus_per_node) || !tlist) {
 		error("there is a problem with your fake_step_layout request\n"
 		      "node_cnt = %u, task_cnt = %u, tlist = %s",
 		      node_cnt, task_cnt, tlist);
 		return NULL;
 	}
 
-	hl = hostlist_create(tlist);
-	/* make out how many cpus there are on each node */
-	if(task_cnt > 0)
-		cpn = (task_cnt + node_cnt - 1) / node_cnt;
-
 	step_layout = xmalloc(sizeof(slurm_step_layout_t));
 	step_layout->node_list = xstrdup(tlist);
 	step_layout->node_cnt = node_cnt;
 	step_layout->tasks = xmalloc(sizeof(uint16_t) * node_cnt);
 	step_layout->tids  = xmalloc(sizeof(uint32_t *) * node_cnt);
-/* 	step_layout->node_addr =  */
-/* 		xmalloc(sizeof(slurm_addr_t) * node_cnt); */
 
 	step_layout->task_cnt = 0;
-	for (i=0; i<step_layout->node_cnt; i++) {
-		if(cpus_per_node && cpu_count_reps) {
+	for (i = 0; i < step_layout->node_cnt; i++) {
+		if (cpus_per_node && cpu_count_reps) {
 			step_layout->tasks[i] = cpus_per_node[cpu_inx];
 			step_layout->tids[i] = xmalloc(sizeof(uint32_t) *
 						       step_layout->tasks[i]);
 
-			for (j=0; j<step_layout->tasks[i]; j++)
+			for (j = 0; j < step_layout->tasks[i]; j++)
 				step_layout->tids[i][j] =
 					step_layout->task_cnt++;
 
@@ -205,7 +196,9 @@ slurm_step_layout_t *fake_slurm_step_layout_create(
 				cpu_cnt = 0;
 			}
 		} else {
-			if(step_layout->task_cnt >= task_cnt) {
+			cpn = ((task_cnt - step_layout->task_cnt) +
+			       (node_cnt - i) - 1) / (node_cnt - i);
+			if (step_layout->task_cnt >= task_cnt) {
 				step_layout->tasks[i] = 0;
 				step_layout->tids[i] = NULL;
 			} else {
@@ -213,36 +206,19 @@ slurm_step_layout_t *fake_slurm_step_layout_create(
 				step_layout->tids[i] =
 					xmalloc(sizeof(uint32_t) * cpn);
 
-				for (j=0; j<cpn; j++) {
+				for (j = 0; j < cpn; j++) {
 					step_layout->tids[i][j] =
 						step_layout->task_cnt++;
-					if(step_layout->task_cnt >= task_cnt) {
+					if (step_layout->task_cnt >= task_cnt) {
 						step_layout->tasks[i] = j + 1;
 						break;
 					}
 				}
 			}
 		}
-/* 		name = hostlist_shift(hl); */
-/* 		if(!name) { */
-/* 			error("fake_slurm_step_layout_create: " */
-/* 			      "We don't have the correct nodelist."); */
-/* 			goto error;			       */
-/* 		} */
-/* 		if(slurm_conf_get_addr(name, &step_layout->node_addr[i]) ==  */
-/* 		   SLURM_ERROR) { */
-/* 			error("fake_slurm_step_layout_create: " */
-/* 			      "we didn't get an addr for host %s.", name); */
-
-/* 		} */
-/* 		free(name); */
 	}
-	hostlist_destroy(hl);
+
 	return step_layout;
-/* error: */
-/* 	hostlist_destroy(hl); */
-/* 	slurm_step_layout_destroy(step_layout); */
-/* 	return NULL; */
 }
 
 
@@ -252,8 +228,8 @@ extern slurm_step_layout_t *slurm_step_layout_copy(
 	slurm_step_layout_t *step_layout)
 {
 	slurm_step_layout_t *layout;
-	int i=0;
-	if(!step_layout)
+	int i = 0;
+	if (!step_layout)
 		return NULL;
 
 	layout = xmalloc(sizeof(slurm_step_layout_t));
@@ -262,16 +238,12 @@ extern slurm_step_layout_t *slurm_step_layout_copy(
 	layout->task_cnt = step_layout->task_cnt;
 	layout->task_dist = step_layout->task_dist;
 
-/* 	layout->node_addr = xmalloc(sizeof(slurm_addr_t) * layout->node_cnt); */
-/* 	memcpy(layout->node_addr, step_layout->node_addr,  */
-/* 	       (sizeof(slurm_addr_t) * layout->node_cnt)); */
-
 	layout->tasks = xmalloc(sizeof(uint16_t) * layout->node_cnt);
 	memcpy(layout->tasks, step_layout->tasks,
 	       (sizeof(uint16_t) * layout->node_cnt));
 
 	layout->tids  = xmalloc(sizeof(uint32_t *) * layout->node_cnt);
-	for (i=0; i<layout->node_cnt; i++) {
+	for (i = 0; i < layout->node_cnt; i++) {
 		layout->tids[i] = xmalloc(sizeof(uint32_t) * layout->tasks[i]);
 		memcpy(layout->tids[i], step_layout->tids[i],
 		       (sizeof(uint32_t) * layout->tasks[i]));
@@ -303,23 +275,9 @@ extern void pack_slurm_step_layout(slurm_step_layout_t *step_layout,
 				     step_layout->tasks[i],
 				     buffer);
 		}
-	} else if (protocol_version >= SLURM_2_1_PROTOCOL_VERSION) {
-		if (step_layout)
-			i=1;
-
-		pack16(i, buffer);
-		if (!i)
-			return;
-		packstr(step_layout->node_list, buffer);
-		pack32(step_layout->node_cnt, buffer);
-		pack32(step_layout->task_cnt, buffer);
-		pack16(step_layout->task_dist, buffer);
-
-		for (i=0; i<step_layout->node_cnt; i++) {
-			pack32_array(step_layout->tids[i],
-				     step_layout->tasks[i],
-				     buffer);
-		}
+	} else {
+		error("pack_slurm_step_layout: protocol_version "
+		      "%hu not supported", protocol_version);
 	}
 }
 
@@ -357,30 +315,10 @@ extern int unpack_slurm_step_layout(slurm_step_layout_t **layout, Buf buffer,
 					    buffer);
 			step_layout->tasks[i] = num_tids;
 		}
-	} else if (protocol_version >= SLURM_2_1_PROTOCOL_VERSION) {
-		safe_unpack16(&uint16_tmp, buffer);
-		if (!uint16_tmp)
-			return SLURM_SUCCESS;
-
-		step_layout = xmalloc(sizeof(slurm_step_layout_t));
-		*layout = step_layout;
-
-		safe_unpackstr_xmalloc(&step_layout->node_list,
-				       &uint32_tmp, buffer);
-		safe_unpack32(&step_layout->node_cnt, buffer);
-		safe_unpack32(&step_layout->task_cnt, buffer);
-		safe_unpack16(&step_layout->task_dist, buffer);
-
-		step_layout->tasks =
-			xmalloc(sizeof(uint32_t) * step_layout->node_cnt);
-		step_layout->tids = xmalloc(sizeof(uint32_t *)
-					    * step_layout->node_cnt);
-		for (i = 0; i < step_layout->node_cnt; i++) {
-			safe_unpack32_array(&(step_layout->tids[i]),
-					    &num_tids,
-					    buffer);
-			step_layout->tasks[i] = num_tids;
-		}
+	} else {
+		error("unpack_slurm_step_layout: protocol_version "
+		      "%hu not supported", protocol_version);
+		goto unpack_error;
 	}
 	return SLURM_SUCCESS;
 
@@ -412,7 +350,7 @@ extern int slurm_step_layout_destroy(slurm_step_layout_t *step_layout)
 int slurm_step_layout_host_id (slurm_step_layout_t *s, int taskid)
 {
 	int i, j;
-	if (taskid > s->task_cnt - 1)
+	if (!s->tasks || !s->tids || (taskid > s->task_cnt - 1))
 		return SLURM_ERROR;
 	for (i=0; i < s->node_cnt; i++)
 		for (j=0; j<s->tasks[i]; j++)
