@@ -69,7 +69,7 @@ typedef enum {
 	REQUEST_STEP_SUSPEND,
 	REQUEST_STEP_RESUME,
 	REQUEST_STEP_TERMINATE,
-	REQUEST_STEP_COMPLETION,      /* See REQUEST_STEP_COMPLETION_V2 below */
+	REQUEST_STEP_COMPLETION,  /* Defunct, See REQUEST_STEP_COMPLETION_V2 */
 	REQUEST_STEP_TASK_INFO,
 	REQUEST_STEP_LIST_PIDS,
 	REQUEST_STEP_RECONFIGURE,
