@@ -95,16 +95,36 @@ int _file_write_content(char* file_path, char* content, size_t csize);
  *  - XCGROUP_SUCCESS
  */
 int xcgroup_ns_create(slurm_cgroup_conf_t *conf,
-		xcgroup_ns_t* cgns, char* mnt_point, char* mnt_args,
-		      char* subsys, char* notify_prog) {
-
-	cgns->mnt_point = xstrdup(conf->cgroup_mountpoint);
-	xstrcat(cgns->mnt_point, mnt_point);
+		      xcgroup_ns_t *cgns, char *mnt_args, char *subsys) {
 
+	cgns->mnt_point = xstrdup_printf("%s/%s",
+					 conf->cgroup_mountpoint, subsys);
 	cgns->mnt_args = xstrdup(mnt_args);
 	cgns->subsystems = xstrdup(subsys);
-	cgns->notify_prog = xstrdup(notify_prog);
+	cgns->notify_prog = xstrdup_printf("%s/release_%s",
+					   conf->cgroup_release_agent, subsys);
+
+	/* check that freezer cgroup namespace is available */
+	if (!xcgroup_ns_is_available(cgns)) {
+		if (conf->cgroup_automount) {
+			if (xcgroup_ns_mount(cgns)) {
+				error("unable to mount %s cgroup "
+				      "namespace: %s",
+				      subsys, slurm_strerror(errno));
+				goto clean;
+			}
+			info("cgroup namespace '%s' is now mounted", subsys);
+		} else {
+			error("cgroup namespace '%s' not mounted. aborting",
+			      subsys);
+			goto clean;
+		}
+	}
+
 	return XCGROUP_SUCCESS;
+clean:
+	xcgroup_ns_destroy(cgns);
+	return XCGROUP_ERROR;
 }
 
 /*
@@ -114,8 +134,8 @@ int xcgroup_ns_create(slurm_cgroup_conf_t *conf,
  *  - XCGROUP_ERROR
  *  - XCGROUP_SUCCESS
  */
-int xcgroup_ns_destroy(xcgroup_ns_t* cgns) {
-
+int xcgroup_ns_destroy(xcgroup_ns_t* cgns)
+{
 	xfree(cgns->mnt_point);
 	xfree(cgns->mnt_args);
 	xfree(cgns->subsystems);
@@ -197,10 +217,20 @@ int xcgroup_ns_mount(xcgroup_ns_t* cgns)
 		options = opt_combined;
 	}
 
+#if defined(__FreeBSD__)
+	if (mount("cgroup", cgns->mnt_point,
+		  MS_NOSUID|MS_NOEXEC|MS_NODEV, options))
+#else
 	if (mount("cgroup", cgns->mnt_point, "cgroup",
 		  MS_NOSUID|MS_NOEXEC|MS_NODEV, options))
+#endif
 		return XCGROUP_ERROR;
 	else {
+		/* FIXME: this only gets set when we aren't mounted at
+		   all.  Since we never umount this may only be loaded
+		   at startup the first time.
+		*/
+
 		/* we then set the release_agent if necessary */
 		if (cgns->notify_prog) {
 			if (xcgroup_create(cgns, &cg, "/", 0, 0) ==
@@ -208,6 +238,7 @@ int xcgroup_ns_mount(xcgroup_ns_t* cgns)
 				return XCGROUP_SUCCESS;
 			xcgroup_set_param(&cg, "release_agent",
 					  cgns->notify_prog);
+			xcgroup_destroy(&cg);
 		}
 		return XCGROUP_SUCCESS;
 	}
@@ -238,16 +269,15 @@ int xcgroup_ns_umount(xcgroup_ns_t* cgns)
  */
 int xcgroup_ns_is_available(xcgroup_ns_t* cgns)
 {
-	int fstatus;
+	int fstatus = 0;
 	char* value;
 	size_t s;
 	xcgroup_t cg;
 
 	if (xcgroup_create(cgns, &cg, "/", 0, 0) == XCGROUP_ERROR)
 		return 0;
-
 	if (xcgroup_get_param(&cg, "release_agent",
-			       &value, &s) != XCGROUP_SUCCESS)
+			      &value, &s) != XCGROUP_SUCCESS)
 		fstatus = 0;
 	else {
 		xfree(value);
@@ -356,6 +386,7 @@ int xcgroup_create(xcgroup_ns_t* cgns, xcgroup_t* cg,
 	cg->path = xstrdup(file_path);
 	cg->uid = uid;
 	cg->gid = gid;
+	cg->notify = 1;
 
 	return XCGROUP_SUCCESS;
 }
@@ -417,7 +448,7 @@ int xcgroup_instanciate(xcgroup_t* cg)
 	uid_t uid;
 	gid_t gid;
 	int create_only;
-	int notify;
+	uint32_t notify;
 
 	/* init variables based on input cgroup */
 	cgns = cg->ns;
@@ -425,7 +456,7 @@ int xcgroup_instanciate(xcgroup_t* cg)
 	uid = cg->uid;
 	gid = cg->gid;
 	create_only=0;
-	notify=1;
+	notify = cg->notify;
 
 	/* save current mask and apply working one */
 	cmask = S_IWGRP | S_IWOTH;
@@ -454,7 +485,7 @@ int xcgroup_instanciate(xcgroup_t* cg)
 	fstatus = XCGROUP_SUCCESS;
 
 	/* set notify on release flag */
-	if (notify && cgns->notify_prog)
+	if (notify == 1 && cgns->notify_prog)
 		xcgroup_set_params(cg, "notify_on_release=1");
 	else
 		xcgroup_set_params(cg, "notify_on_release=0");
@@ -489,6 +520,9 @@ int xcgroup_load(xcgroup_ns_t* cgns, xcgroup_t* cg, char* uri)
 	cg->uid = buf.st_uid;
 	cg->gid = buf.st_gid;
 
+	/* read the content of the notify flag */
+	xcgroup_get_uint32_param(cg,"notify_on_release",&(cg->notify));
+
 	return XCGROUP_SUCCESS;
 }
 
@@ -872,7 +906,7 @@ int _file_write_uint64s(char* file_path, uint64_t* values, int nb)
 		do {
 			rc = write(fd, tstr, strlen(tstr)+1);
 		}
-		while (rc != 0 && errno == EINTR);
+		while (rc < 0 && errno == EINTR);
 		if (rc < 1) {
 			debug2("unable to add value '%s' to file '%s' : %m",
 			       tstr, file_path);
@@ -1097,7 +1131,7 @@ int _file_write_content(char* file_path, char* content, size_t csize)
 	do {
 		rc = write(fd, content, csize);
 	}
-	while (rc != 0 && errno == EINTR);
+	while (rc < 0 && errno == EINTR);
 
 	/* check read size */
 	if (rc < csize) {
