@@ -52,9 +52,9 @@ extern List setup_cluster_list_with_inx(mysql_conn_t *mysql_conn,
 extern int good_nodes_from_inx(List local_cluster_list,
 			       void **object, char *node_inx,
 			       int submit);
-extern char *setup_job_cluster_cond_limits(mysql_conn_t *mysql_conn,
-					   slurmdb_job_cond_t *job_cond,
-					   char *cluster_name, char **extra);
+extern int setup_job_cluster_cond_limits(mysql_conn_t *mysql_conn,
+					 slurmdb_job_cond_t *job_cond,
+					 char *cluster_name, char **extra);
 extern int setup_job_cond_limits(mysql_conn_t *mysql_conn,
 				 slurmdb_job_cond_t *job_cond,
 				 char **extra);
