@@ -105,6 +105,7 @@ enum local_error_code {
 	ESIG_BUF_DATA_MISMATCH = 5000,
 	ESIG_BUF_SIZE_MISMATCH,
 	ESIG_BAD_USERID,
+	ESIG_CRED_REPLAYED,
 };
 
 static uid_t slurm_user = 0;
@@ -187,6 +188,8 @@ crypto_str_error(int errnum)
 		return "Credential data size mismatch";
 	else if (errnum == ESIG_BAD_USERID)
 		return "Credential created by invalid user";
+	else if (errnum == ESIG_CRED_REPLAYED)
+		return "Credential replayed";
 	else
 		return munge_strerror ((munge_err_t) errnum);
 }
@@ -196,14 +199,21 @@ extern int
 crypto_sign(void * key, char *buffer, int buf_size, char **sig_pp,
 	    unsigned int *sig_size_p)
 {
+	int retry = 2;
 	char *cred;
 	munge_err_t err;
 
-	err = munge_encode(&cred, (munge_ctx_t) key,
-			   buffer, buf_size);
-
-	if (err != EMUNGE_SUCCESS)
+    again:
+	err = munge_encode(&cred, (munge_ctx_t) key, buffer, buf_size);
+	if (err != EMUNGE_SUCCESS) {
+		if ((err == EMUNGE_SOCKET) && retry--) {
+#ifdef MULTIPLE_SLURMD
+			sleep(1);
+#endif
+			goto again;
+		}
 		return err;
+	}
 
 	*sig_size_p = strlen(cred) + 1;
 	*sig_pp = xstrdup(cred);
@@ -215,6 +225,7 @@ extern int
 crypto_verify_sign(void * key, char *buffer, unsigned int buf_size,
 		   char *signature, unsigned int sig_size)
 {
+	int retry = 2;
 	uid_t uid;
 	gid_t gid;
 	void *buf_out = NULL;
@@ -222,26 +233,34 @@ crypto_verify_sign(void * key, char *buffer, unsigned int buf_size,
 	int   rc = 0;
 	munge_err_t err;
 
+    again:
 	err = munge_decode(signature, (munge_ctx_t) key,
 			   &buf_out, &buf_out_size,
 			   &uid, &gid);
 
 	if (err != EMUNGE_SUCCESS) {
+		if ((err == EMUNGE_SOCKET) && retry--) {
+			error ("Munge decode failed: %s (retrying ...)",
+				munge_ctx_strerror((munge_ctx_t) key));
+#ifdef MULTIPLE_SLURMD
+			sleep(1);
+#endif
+			goto again;
+		}
+
 #ifdef MULTIPLE_SLURMD
-		/* In multple slurmd mode this will happen all the
-		 * time since we are authenticating with the same
-		 * munged.
-		 */
 		if (err != EMUNGE_CRED_REPLAYED) {
 			rc = err;
 			goto end_it;
 		} else {
-			debug2("We had a replayed crypto, "
-			       "but this is expected in multiple "
-			       "slurmd mode.");
+			debug2("We had a replayed crypto, but this "
+			       "is expected in multiple slurmd mode.");
 		}
 #else
-		rc = err;
+		if (err == EMUNGE_CRED_REPLAYED)
+			rc = ESIG_CRED_REPLAYED;
+		else
+			rc = err;
 		goto end_it;
 #endif
 	}
