@@ -105,13 +105,13 @@ const uint32_t min_plug_version = 100;
  * plugin. If you develop another plugin that may be of interest to others
  * please post it to slurm-dev@lists.llnl.gov  Thanks!
 \*****************************************************************************/
-
-/* This example code will prevent users from setting an accounting frequency
- * of less than 30 seconds in order to insure more precise accounting.
- * Also remove any QOS value set by the user in order to use the default value
- * from the database. */
 extern int job_submit(struct job_descriptor *job_desc, uint32_t submit_uid)
 {
+#if 0
+	/* This example code will prevent users from setting an accounting
+	 * frequency of less than 30 seconds in order to insure more precise
+	 *  accounting. Also remove any QOS value set by the user in order
+	 * to use the default value from the database. */
 	if (job_desc->acctg_freq < MIN_ACCTG_FREQUENCY) {
 		info("Changing accounting frequency of submitted job "
 		     "from %u to %u",
@@ -123,17 +123,18 @@ extern int job_submit(struct job_descriptor *job_desc, uint32_t submit_uid)
 		info("Clearing QOS (%s) from submitted job", job_desc->qos);
 		xfree(job_desc->qos);
 	}
-
+#endif
 	return SLURM_SUCCESS;
 }
 
-/* This example code will prevent users from setting an accounting frequency
- * of less than 30 seconds in order to insure more precise accounting.
- * Also remove any QOS value set by the user in order to use the default value
- * from the database. */
 extern int job_modify(struct job_descriptor *job_desc,
 		      struct job_record *job_ptr, uint32_t submit_uid)
 {
+#if 0
+	/* This example code will prevent users from setting an accounting
+	 * frequency of less than 30 seconds in order to insure more precise
+	 *  accounting. Also remove any QOS value set by the user in order
+	 * to use the default value from the database. */
 	if (job_desc->acctg_freq < MIN_ACCTG_FREQUENCY) {
 		info("Changing accounting frequency of modify job %u "
 		     "from %u to %u", job_ptr->job_id,
@@ -146,6 +147,6 @@ extern int job_modify(struct job_descriptor *job_desc,
 		     job_desc->qos, job_ptr->job_id);
 		xfree(job_desc->qos);
 	}
-
+#endif
 	return SLURM_SUCCESS;
 }
