@@ -50,6 +50,7 @@
 #include "src/common/slurm_xlator.h"
 #include "src/common/mpi.h"
 #include "src/common/env.h"
+#include "src/slurmd/slurmstepd/slurmstepd_job.h"
 
 /*
  * These variables are required by the generic plugin interface.  If they
@@ -82,6 +83,12 @@ const char plugin_name[]        = "mpi none plugin";
 const char plugin_type[]        = "mpi/none";
 const uint32_t plugin_version   = 100;
 
+int p_mpi_hook_slurmstepd_prefork(const slurmd_job_t *job, char ***env)
+{
+	debug("mpi/none: slurmstepd prefork");
+	return SLURM_SUCCESS;
+}
+
 int p_mpi_hook_slurmstepd_task(const mpi_plugin_task_info_t*job,
 			       char ***env)
 {
@@ -97,7 +104,7 @@ p_mpi_hook_client_prelaunch(const mpi_plugin_client_info_t *job, char ***env)
 	return (void *)0xdeadbeef;
 }
 
-int p_mpi_hook_client_single_task_per_node()
+int p_mpi_hook_client_single_task_per_node(void)
 {
 	return false;
 }
