@@ -64,6 +64,7 @@
 #include "src/common/macros.h"
 #include "src/common/node_select.h"
 #include "src/common/parse_time.h"
+#include "src/common/slurm_accounting_storage.h"
 #include "src/common/slurm_protocol_api.h"
 #include "src/common/xmalloc.h"
 #include "src/common/xstring.h"
@@ -111,7 +112,7 @@ typedef struct node_space_map {
 
 /* Diag statistics */
 extern diag_stats_t slurmctld_diag_stats;
-int bf_last_ints = 0;
+int bf_last_yields = 0;
 
 /*********************** local variables *********************/
 static bool stop_backfill = false;
@@ -146,7 +147,7 @@ static bool _test_resv_overlap(node_space_map_t *node_space,
 			       uint32_t end_reserve);
 static int  _try_sched(struct job_record *job_ptr, bitstr_t **avail_bitmap,
 		       uint32_t min_nodes, uint32_t max_nodes,
-		       uint32_t req_nodes);
+		       uint32_t req_nodes, bitstr_t *exc_core_bitmap);
 
 /* Log resource allocate table */
 static void _dump_node_space_table(node_space_map_t *node_space_ptr)
@@ -240,16 +241,18 @@ static int _num_feature_count(struct job_record *job_ptr)
 /* Attempt to schedule a specific job on specific available nodes
  * IN job_ptr - job to schedule
  * IN/OUT avail_bitmap - nodes available/selected to use
+ * IN exc_core_bitmap - cores which can not be used
  * RET SLURM_SUCCESS on success, otherwise an error code
  */
 static int  _try_sched(struct job_record *job_ptr, bitstr_t **avail_bitmap,
 		       uint32_t min_nodes, uint32_t max_nodes,
-		       uint32_t req_nodes)
+		       uint32_t req_nodes, bitstr_t *exc_core_bitmap)
 {
 	bitstr_t *tmp_bitmap;
 	int rc = SLURM_SUCCESS;
 	int feat_cnt = _num_feature_count(job_ptr);
 	List preemptee_candidates = NULL;
+	List preemptee_job_list = NULL;
 
 	if (feat_cnt) {
 		/* Ideally schedule the job feature by feature,
@@ -289,7 +292,13 @@ static int  _try_sched(struct job_record *job_ptr, bitstr_t **avail_bitmap,
 			rc = select_g_job_test(job_ptr, *avail_bitmap,
 					       high_cnt, max_nodes, req_nodes,
 					       SELECT_MODE_WILL_RUN,
-					       preemptee_candidates, NULL);
+					       preemptee_candidates,
+					       &preemptee_job_list,
+					       exc_core_bitmap);
+			if (preemptee_job_list) {
+				list_destroy(preemptee_job_list);
+				preemptee_job_list = NULL;
+			}
 		}
 
 		/* Restore the feature counts */
@@ -306,15 +315,31 @@ static int  _try_sched(struct job_record *job_ptr, bitstr_t **avail_bitmap,
 		 * then on shared nodes (if so configured). */
 		uint16_t orig_shared;
 		time_t now = time(NULL);
+		char str[100];
+
 		preemptee_candidates = slurm_find_preemptable_jobs(job_ptr);
 		orig_shared = job_ptr->details->shared;
 		job_ptr->details->shared = 0;
 		tmp_bitmap = bit_copy(*avail_bitmap);
+
+		if (exc_core_bitmap) {
+			bit_fmt(str, (sizeof(str) - 1), exc_core_bitmap);
+			debug2(" _try_sched with exclude core bitmap: %s",str);
+		}
+
 		rc = select_g_job_test(job_ptr, *avail_bitmap, min_nodes,
 				       max_nodes, req_nodes,
 				       SELECT_MODE_WILL_RUN,
-				       preemptee_candidates, NULL);
+				       preemptee_candidates,
+				       &preemptee_job_list,
+				       exc_core_bitmap);
+		if (preemptee_job_list) {
+			list_destroy(preemptee_job_list);
+			preemptee_job_list = NULL;
+		}
+
 		job_ptr->details->shared = orig_shared;
+
 		if (((rc != SLURM_SUCCESS) || (job_ptr->start_time > now)) &&
 		    (orig_shared != 0)) {
 			FREE_NULL_BITMAP(*avail_bitmap);
@@ -322,7 +347,13 @@ static int  _try_sched(struct job_record *job_ptr, bitstr_t **avail_bitmap,
 			rc = select_g_job_test(job_ptr, *avail_bitmap,
 					       min_nodes, max_nodes, req_nodes,
 					       SELECT_MODE_WILL_RUN,
-					       preemptee_candidates, NULL);
+					       preemptee_candidates,
+					       &preemptee_job_list,
+					       exc_core_bitmap);
+			if (preemptee_job_list) {
+				list_destroy(preemptee_job_list);
+				preemptee_job_list = NULL;
+			}
 		} else
 			FREE_NULL_BITMAP(tmp_bitmap);
 	}
@@ -405,22 +436,24 @@ extern void backfill_reconfig(void)
 	config_flag = true;
 }
 
-static void _do_diag_stats(struct timeval *tv1, struct timeval *tv2)
+static void _do_diag_stats(struct timeval *tv1, struct timeval *tv2,
+			   int yield_sleep)
 {
-	long delta_t;
-	long bf_interval_usecs = backfill_interval * 1000000;
+	uint32_t yield_sleep_usecs = yield_sleep * 1000000;
+	uint32_t delta_t, real_time;
 
 	delta_t  = (tv2->tv_sec  - tv1->tv_sec) * 1000000;
 	delta_t +=  tv2->tv_usec - tv1->tv_usec;
 
+	real_time = (delta_t - (bf_last_yields * yield_sleep_usecs));
+
 	slurmctld_diag_stats.bf_cycle_counter++;
-	slurmctld_diag_stats.bf_cycle_sum += (delta_t -(bf_last_ints *
-							bf_interval_usecs));
-   	slurmctld_diag_stats.bf_cycle_last = delta_t - (bf_last_ints *
-							bf_interval_usecs);
+	slurmctld_diag_stats.bf_cycle_sum += real_time;
+	slurmctld_diag_stats.bf_cycle_last = real_time;
+
 	slurmctld_diag_stats.bf_depth_sum += slurmctld_diag_stats.bf_last_depth;
-	slurmctld_diag_stats.bf_depth_try_sum += slurmctld_diag_stats.
-						 bf_last_depth_try;
+	slurmctld_diag_stats.bf_depth_try_sum +=
+		slurmctld_diag_stats.bf_last_depth_try;
 	if (slurmctld_diag_stats.bf_cycle_last >
 	    slurmctld_diag_stats.bf_cycle_max) {
 		slurmctld_diag_stats.bf_cycle_max = slurmctld_diag_stats.
@@ -459,7 +492,7 @@ extern void *backfill_agent(void *args)
 			continue;
 
 		lock_slurmctld(all_locks);
-		while (_attempt_backfill()) ;
+		(void) _attempt_backfill();
 		last_backfill_time = time(NULL);
 		unlock_slurmctld(all_locks);
 	}
@@ -479,7 +512,7 @@ static int _yield_locks(int secs)
 	part_update = last_part_update;
 
 	unlock_slurmctld(all_locks);
-	bf_last_ints++;
+	bf_last_yields++;
 	_my_sleep(secs);
 	lock_slurmctld(all_locks);
 
@@ -506,16 +539,18 @@ static int _attempt_backfill(void)
 	uint32_t time_limit, comp_time_limit, orig_time_limit;
 	uint32_t min_nodes, max_nodes, req_nodes;
 	bitstr_t *avail_bitmap = NULL, *resv_bitmap = NULL;
-	time_t now, sched_start, later_start, start_res;
+	bitstr_t *exc_core_bitmap = NULL;
+	time_t now, sched_start, later_start, start_res, resv_end;
 	node_space_map_t *node_space;
 	struct timeval bf_time1, bf_time2;
-	static int sched_timeout = 0;
-	int this_sched_timeout = 0, rc = 0;
+	int sched_timeout = 2, yield_sleep = 1;
+	int rc = 0;
 	int job_test_count = 0;
 	uint32_t *uid = NULL, nuser = 0;
 	uint16_t *njobs = NULL;
 	bool already_counted;
 
+	bf_last_yields = 0;
 #ifdef HAVE_CRAY
 	/*
 	 * Run a Basil Inventory immediately before setting up the schedule
@@ -540,12 +575,6 @@ static int _attempt_backfill(void)
 	if (debug_flags & DEBUG_FLAG_BACKFILL)
 		info("backfill: beginning");
 	sched_start = now = time(NULL);
-	if (sched_timeout == 0) {
-		sched_timeout = slurm_get_msg_timeout() / 2;
-		sched_timeout = MAX(sched_timeout, 1);
-		sched_timeout = MIN(sched_timeout, 10);
-	}
-	this_sched_timeout = sched_timeout;
 
 	if (slurm_get_root_filter())
 		filter_root = true;
@@ -565,7 +594,6 @@ static int _attempt_backfill(void)
 	slurmctld_diag_stats.bf_last_depth = 0;
 	slurmctld_diag_stats.bf_last_depth_try = 0;
 	slurmctld_diag_stats.bf_when_last_cycle = now;
-	bf_last_ints = 0;
 	slurmctld_diag_stats.bf_active = 1;
 
 	node_space = xmalloc(sizeof(node_space_map_t) *
@@ -584,9 +612,37 @@ static int _attempt_backfill(void)
 	}
 	while ((job_queue_rec = (job_queue_rec_t *)
 				list_pop_bottom(job_queue, sort_job_queue2))) {
-		job_test_count++;
 		job_ptr  = job_queue_rec->job_ptr;
+		orig_time_limit = job_ptr->time_limit;
+
+		if ((time(NULL) - sched_start) >= sched_timeout) {
+			uint32_t save_time_limit = job_ptr->time_limit;
+			job_ptr->time_limit = orig_time_limit;
+			if (debug_flags & DEBUG_FLAG_BACKFILL) {
+				END_TIMER;
+				info("backfill: completed yielding locks "
+				     "after testing %d jobs, %s",
+				     job_test_count, TIME_STR);
+			}
+			if (_yield_locks(yield_sleep)) {
+				if (debug_flags & DEBUG_FLAG_BACKFILL) {
+					info("backfill: system state changed, "
+					     "breaking out after testing %d "
+					     "jobs", job_test_count);
+				}
+				rc = 1;
+				break;
+			}
+			job_ptr->time_limit = save_time_limit;
+			/* Reset backfill scheduling timers, resume testing */
+			sched_start = time(NULL);
+			job_test_count = 0;
+			START_TIMER;
+		}
+
 		part_ptr = job_queue_rec->part_ptr;
+		job_test_count++;
+
 		xfree(job_queue_rec);
 		if (!IS_JOB_PENDING(job_ptr))
 			continue;	/* started in other partition */
@@ -602,8 +658,10 @@ static int _attempt_backfill(void)
 			for (j = 0; j < nuser; j++) {
 				if (job_ptr->user_id == uid[j]) {
 					njobs[j]++;
-					debug2("backfill: user %u: #jobs %u",
-					       uid[j], njobs[j]);
+					if (debug_flags & DEBUG_FLAG_BACKFILL)
+						debug("backfill: user %u: "
+						      "#jobs %u",
+						      uid[j], njobs[j]);
 					break;
 				}
 			}
@@ -617,18 +675,21 @@ static int _attempt_backfill(void)
 					      "queue. Consider increasing "
 					      "BF_MAX_USERS");
 				}
-				debug2("backfill: found new user %u. "
-				       "Total #users now %u",
-				       job_ptr->user_id, nuser);
+				if (debug_flags & DEBUG_FLAG_BACKFILL)
+					debug2("backfill: found new user %u. "
+					       "Total #users now %u",
+					       job_ptr->user_id, nuser);
 			} else {
 				if (njobs[j] > max_backfill_job_per_user) {
 					/* skip job */
-					debug("backfill: have already checked "
-					      "%u jobs for user %u; skipping "
-					      "job %u",
-					      max_backfill_job_per_user,
-					      job_ptr->user_id,
-					      job_ptr->job_id);
+					if (debug_flags & DEBUG_FLAG_BACKFILL)
+						debug("backfill: have already "
+						      "checked %u jobs for "
+						      "user %u; skipping "
+						      "job %u",
+						      max_backfill_job_per_user,
+						      job_ptr->user_id,
+						      job_ptr->job_id);
 					continue;
 				}
 			}
@@ -676,19 +737,47 @@ static int _attempt_backfill(void)
 						 part_ptr->max_time);
 		}
 		comp_time_limit = time_limit;
-		orig_time_limit = job_ptr->time_limit;
 		qos_ptr = job_ptr->qos_ptr;
-		if (qos_ptr && (qos_ptr->flags & QOS_FLAG_NO_RESERVE))
+		if (qos_ptr && (qos_ptr->flags & QOS_FLAG_NO_RESERVE) &&
+		    slurm_get_preempt_mode())
 			time_limit = job_ptr->time_limit = 1;
 		else if (job_ptr->time_min && (job_ptr->time_min < time_limit))
 			time_limit = job_ptr->time_limit = job_ptr->time_min;
 
 		/* Determine impact of any resource reservations */
 		later_start = now;
- TRY_LATER:	FREE_NULL_BITMAP(avail_bitmap);
+ TRY_LATER:
+		if ((time(NULL) - sched_start) >= sched_timeout) {
+			uint32_t save_time_limit = job_ptr->time_limit;
+			job_ptr->time_limit = orig_time_limit;
+			if (debug_flags & DEBUG_FLAG_BACKFILL) {
+				END_TIMER;
+				info("backfill: completed yielding locks 2"
+				     "after testing %d jobs, %s",
+				     job_test_count, TIME_STR);
+			}
+			if (_yield_locks(yield_sleep)) {
+				if (debug_flags & DEBUG_FLAG_BACKFILL) {
+					info("backfill: system state changed, "
+					     "breaking out after testing %d "
+					     "jobs", job_test_count);
+				}
+				rc = 1;
+				break;
+			}
+			job_ptr->time_limit = save_time_limit;
+			/* Reset backfill scheduling timers, resume testing */
+			sched_start = time(NULL);
+			job_test_count = 1;
+			START_TIMER;
+		}
+
+		FREE_NULL_BITMAP(avail_bitmap);
+		FREE_NULL_BITMAP(exc_core_bitmap);
 		start_res   = later_start;
 		later_start = 0;
-		j = job_test_resv(job_ptr, &start_res, true, &avail_bitmap);
+		j = job_test_resv(job_ptr, &start_res, true, &avail_bitmap,
+				  &exc_core_bitmap);
 		if (j != SLURM_SUCCESS) {
 			job_ptr->time_limit = orig_time_limit;
 			continue;
@@ -697,7 +786,7 @@ static int _attempt_backfill(void)
 			end_time = (time_limit * 60) + start_res;
 		else
 			end_time = (time_limit * 60) + now;
-
+		resv_end = find_resv_end(start_res);
 		/* Identify usable nodes for this job */
 		bit_and(avail_bitmap, part_ptr->node_bitmap);
 		bit_and(avail_bitmap, up_node_bitmap);
@@ -715,6 +804,10 @@ static int _attempt_backfill(void)
 			if ((j = node_space[j].next) == 0)
 				break;
 		}
+		if ((resv_end++) &&
+		    ((later_start == 0) || (resv_end < later_start))) {
+			later_start = resv_end;
+		}
 
 		if (job_ptr->details->exc_node_bitmap) {
 			bit_not(job_ptr->details->exc_node_bitmap);
@@ -746,30 +839,6 @@ static int _attempt_backfill(void)
 		resv_bitmap = bit_copy(avail_bitmap);
 		bit_not(resv_bitmap);
 
-		if ((time(NULL) - sched_start) >= this_sched_timeout) {
-			uint32_t save_time_limit = job_ptr->time_limit;
-			job_ptr->time_limit = orig_time_limit;
-			if (debug_flags & DEBUG_FLAG_BACKFILL) {
-				END_TIMER;
-				info("backfill: yielding locks after testing "
-				     "%d jobs, %s",
-				     job_test_count, TIME_STR);
-			}
-			if (_yield_locks(backfill_interval)) {
-				if (debug_flags & DEBUG_FLAG_BACKFILL) {
-					info("backfill: system state changed, "
-					     "breaking out after testing %d "
-					     "jobs", job_test_count);
-				}
-				rc = 1;
-				break;
-			}
-			job_ptr->time_limit = save_time_limit;
-			/* Reset backfill scheduling timers, resume testing */
-			sched_start = time(NULL);
-			job_test_count = 0;
-			START_TIMER;
-		}
 		/* this is the time consuming operation */
 		debug2("backfill: entering _try_sched for job %u.",
 		       job_ptr->job_id);
@@ -779,14 +848,13 @@ static int _attempt_backfill(void)
 			already_counted = true;
 		}
 
-		j = _try_sched(job_ptr, &avail_bitmap,
-			       min_nodes, max_nodes, req_nodes);
-		debug2("backfill: finished _try_sched for job %u.",
-		       job_ptr->job_id);
+		j = _try_sched(job_ptr, &avail_bitmap, min_nodes, max_nodes,
+			       req_nodes, exc_core_bitmap);
+
 		now = time(NULL);
 		if (j != SLURM_SUCCESS) {
 			job_ptr->time_limit = orig_time_limit;
-			job_ptr->start_time = 0;	
+			job_ptr->start_time = 0;
 			continue;	/* not runable */
 		}
 
@@ -795,13 +863,16 @@ static int _attempt_backfill(void)
 			last_job_update = now;
 		}
 		if (job_ptr->start_time <= now) {
+			uint32_t save_time_limit = job_ptr->time_limit;
 			int rc = _start_job(job_ptr, resv_bitmap);
-			if (qos_ptr && (qos_ptr->flags & QOS_FLAG_NO_RESERVE)){
-				job_ptr->time_limit = orig_time_limit;
+			if (qos_ptr && (qos_ptr->flags & QOS_FLAG_NO_RESERVE)) {
+				if (orig_time_limit == NO_VAL)
+					job_ptr->time_limit = comp_time_limit;
+				else
+					job_ptr->time_limit = orig_time_limit;
 				job_ptr->end_time = job_ptr->start_time +
-						    (orig_time_limit * 60);
-			}
-			else if ((rc == SLURM_SUCCESS) && job_ptr->time_min) {
+						    (job_ptr->time_limit * 60);
+			} else if ((rc == SLURM_SUCCESS) && job_ptr->time_min) {
 				/* Set time limit as high as possible */
 				job_ptr->time_limit = comp_time_limit;
 				job_ptr->end_time = job_ptr->start_time +
@@ -814,15 +885,19 @@ static int _attempt_backfill(void)
 			}
 			if (rc == ESLURM_ACCOUNTING_POLICY) {
 				/* Unknown future start time, just skip job */
-				job_ptr->start_time = 0;	
+				job_ptr->start_time = 0;
 				continue;
 			} else if (rc != SLURM_SUCCESS) {
 				/* Planned to start job, but something bad
 				 * happended. */
-				job_ptr->start_time = 0;	
+				job_ptr->start_time = 0;
 				break;
 			} else {
-				/* Started this job, move to next one */
+				/* Started this job. Update the database if job
+				 * time limit changed and move to next job */
+				if (save_time_limit != job_ptr->time_limit)
+					jobacct_storage_g_job_start(acct_db_conn,
+								    job_ptr);
 				continue;
 			}
 		} else
@@ -870,6 +945,7 @@ static int _attempt_backfill(void)
 	xfree(uid);
 	xfree(njobs);
 	FREE_NULL_BITMAP(avail_bitmap);
+	FREE_NULL_BITMAP(exc_core_bitmap);
 	FREE_NULL_BITMAP(resv_bitmap);
 
 	for (i=0; ; ) {
@@ -880,7 +956,7 @@ static int _attempt_backfill(void)
 	xfree(node_space);
 	list_destroy(job_queue);
 	gettimeofday(&bf_time2, NULL);
-	_do_diag_stats(&bf_time1, &bf_time2);
+	_do_diag_stats(&bf_time1, &bf_time2, yield_sleep);
 	if (debug_flags & DEBUG_FLAG_BACKFILL) {
 		END_TIMER;
 		info("backfill: completed testing %d jobs, %s",
