@@ -129,6 +129,7 @@ static char *	_will_run_test(uint32_t jobid, time_t start_time,
 	struct job_record *job_ptr = NULL;
 	struct part_record *part_ptr;
 	bitstr_t *avail_bitmap = NULL, *resv_bitmap = NULL;
+	bitstr_t *exc_core_bitmap = NULL;
 	char *hostlist, *reply_msg = NULL;
 	uint32_t min_nodes, max_nodes, req_nodes;
 	int rc;
@@ -173,12 +174,14 @@ static char *	_will_run_test(uint32_t jobid, time_t start_time,
 
 	/* Enforce reservation: access control, time and nodes */
 	start_res = start_time;
-	rc = job_test_resv(job_ptr, &start_res, true, &resv_bitmap);
+	rc = job_test_resv(job_ptr, &start_res, true, &resv_bitmap,
+			   &exc_core_bitmap);
 	if (rc != SLURM_SUCCESS) {
 		*err_code = -730;
 		*err_msg = "Job denied access to reservation";
 		error("wiki: reservation access denied for job %u", jobid);
 		FREE_NULL_BITMAP(avail_bitmap);
+		FREE_NULL_BITMAP(exc_core_bitmap);
 		return NULL;
 	}
 	start_time = MAX(start_time, start_res);
@@ -197,6 +200,7 @@ static char *	_will_run_test(uint32_t jobid, time_t start_time,
 		error("wiki: no nodes in partition %s for job %u",
 			part_ptr->name, jobid);
 		FREE_NULL_BITMAP(avail_bitmap);
+		FREE_NULL_BITMAP(exc_core_bitmap);
 		return NULL;
 	}
 
@@ -208,6 +212,7 @@ static char *	_will_run_test(uint32_t jobid, time_t start_time,
 		error("wiki: job %u not runnable on hosts=%s",
 		      jobid, node_list);
 		FREE_NULL_BITMAP(avail_bitmap);
+		FREE_NULL_BITMAP(exc_core_bitmap);
 		return NULL;
 	}
 	if (job_ptr->details->exc_node_bitmap) {
@@ -223,6 +228,7 @@ static char *	_will_run_test(uint32_t jobid, time_t start_time,
 		error("wiki: job %u not runnable on hosts=%s",
 		      jobid, node_list);
 		FREE_NULL_BITMAP(avail_bitmap);
+		FREE_NULL_BITMAP(exc_core_bitmap);
 		return NULL;
 	}
 
@@ -244,6 +250,7 @@ static char *	_will_run_test(uint32_t jobid, time_t start_time,
 		error("wiki: job %u not runnable on hosts=%s",
 		      jobid, node_list);
 		FREE_NULL_BITMAP(avail_bitmap);
+		FREE_NULL_BITMAP(exc_core_bitmap);
 		return NULL;
 	}
 
@@ -253,7 +260,7 @@ static char *	_will_run_test(uint32_t jobid, time_t start_time,
 	rc = select_g_job_test(job_ptr, avail_bitmap,
 			       min_nodes, max_nodes, req_nodes,
 			       SELECT_MODE_WILL_RUN,
-			       preemptee_candidates, NULL);
+			       preemptee_candidates, NULL, exc_core_bitmap);
 	if (preemptee_candidates)
 		list_destroy(preemptee_candidates);
 
@@ -285,6 +292,7 @@ static char *	_will_run_test(uint32_t jobid, time_t start_time,
 	/* Restore pending job's expected start time */
 	job_ptr->start_time = orig_start_time;
 	FREE_NULL_BITMAP(avail_bitmap);
+	FREE_NULL_BITMAP(exc_core_bitmap);
 	return reply_msg;
 }
 
@@ -412,6 +420,7 @@ static char *	_will_run_test2(uint32_t jobid, time_t start_time,
 	struct job_record *job_ptr = NULL, *pre_ptr;
 	struct part_record *part_ptr;
 	bitstr_t *avail_bitmap = NULL, *resv_bitmap = NULL;
+	bitstr_t *exc_core_bitmap = NULL;
 	time_t start_res;
 	uint32_t min_nodes, max_nodes, req_nodes;
 	List preemptee_candidates = NULL, preempted_jobs = NULL;
@@ -455,12 +464,14 @@ static char *	_will_run_test2(uint32_t jobid, time_t start_time,
 
 	/* Enforce reservation: access control, time and nodes */
 	start_res = start_time;
-	rc = job_test_resv(job_ptr, &start_res, true, &resv_bitmap);
+	rc = job_test_resv(job_ptr, &start_res, true, &resv_bitmap,
+			   &exc_core_bitmap);
 	if (rc != SLURM_SUCCESS) {
 		*err_code = -730;
 		*err_msg = "Job denied access to reservation";
 		error("wiki: reservation access denied for job %u", jobid);
 		FREE_NULL_BITMAP(avail_bitmap);
+		FREE_NULL_BITMAP(exc_core_bitmap);
 		return NULL;
 	}
 	start_time = MAX(start_time, start_res);
@@ -479,6 +490,7 @@ static char *	_will_run_test2(uint32_t jobid, time_t start_time,
 		error("wiki: no nodes in partition %s for job %u",
 			part_ptr->name, jobid);
 		FREE_NULL_BITMAP(avail_bitmap);
+		FREE_NULL_BITMAP(exc_core_bitmap);
 		return NULL;
 	}
 
@@ -490,6 +502,7 @@ static char *	_will_run_test2(uint32_t jobid, time_t start_time,
 		error("wiki: job %u not runnable on hosts=%s",
 			jobid, node_list);
 		FREE_NULL_BITMAP(avail_bitmap);
+		FREE_NULL_BITMAP(exc_core_bitmap);
 		return NULL;
 	}
 	if (job_ptr->details->exc_node_bitmap) {
@@ -505,6 +518,7 @@ static char *	_will_run_test2(uint32_t jobid, time_t start_time,
 		error("wiki: job %u not runnable on hosts=%s",
 			jobid, node_list);
 		FREE_NULL_BITMAP(avail_bitmap);
+		FREE_NULL_BITMAP(exc_core_bitmap);
 		return NULL;
 	}
 
@@ -526,6 +540,7 @@ static char *	_will_run_test2(uint32_t jobid, time_t start_time,
 		error("wiki: job %u not runnable on hosts=%s",
 			jobid, node_list);
 		FREE_NULL_BITMAP(avail_bitmap);
+		FREE_NULL_BITMAP(exc_core_bitmap);
 		return NULL;
 	}
 
@@ -540,7 +555,8 @@ static char *	_will_run_test2(uint32_t jobid, time_t start_time,
 	orig_start_time = job_ptr->start_time;
 	rc = select_g_job_test(job_ptr, avail_bitmap, min_nodes, max_nodes,
 			       req_nodes, SELECT_MODE_WILL_RUN,
-			       preemptee_candidates, &preempted_jobs);
+			       preemptee_candidates, &preempted_jobs,
+			       exc_core_bitmap);
 	if (preemptee_candidates)
 		list_destroy(preemptee_candidates);
 
@@ -584,6 +600,7 @@ static char *	_will_run_test2(uint32_t jobid, time_t start_time,
 	job_ptr->start_time = orig_start_time;
 
 	FREE_NULL_BITMAP(avail_bitmap);
+	FREE_NULL_BITMAP(exc_core_bitmap);
 	return reply_msg;
 }
 
