@@ -26,6 +26,7 @@ static int _set_select_jobinfo(select_jobinfo_t *jobinfo,
 			       enum select_jobdata_type data_type, void *data)
 {
 	uint32_t *uint32 = (uint32_t *) data;
+	uint8_t  *uint8  = (uint8_t *)  data;
 
 	if (jobinfo == NULL) {
 		error("cray/set_select_jobinfo: jobinfo not set");
@@ -37,6 +38,9 @@ static int _set_select_jobinfo(select_jobinfo_t *jobinfo,
 	}
 
 	switch (data_type) {
+	case SELECT_JOBDATA_CONFIRMED:
+		jobinfo->confirmed = *uint8;
+		break;
 	case SELECT_JOBDATA_RESV_ID:
 		jobinfo->reservation_id = *uint32;
 		break;
@@ -53,6 +57,7 @@ static int _get_select_jobinfo(select_jobinfo_t *jobinfo,
 {
 	uint64_t *uint64 = (uint64_t *) data;
 	uint32_t *uint32 = (uint32_t *) data;
+	uint8_t  *uint8  = (uint8_t *)  data;
 
 	if (jobinfo == NULL) {
 		error("cray/get_select_jobinfo: jobinfo not set");
@@ -64,6 +69,9 @@ static int _get_select_jobinfo(select_jobinfo_t *jobinfo,
 	}
 
 	switch (data_type) {
+	case SELECT_JOBDATA_CONFIRMED:
+		*uint8 = jobinfo->confirmed;
+		break;
 	case SELECT_JOBDATA_RESV_ID:
 		*uint32 = jobinfo->reservation_id;
 		break;
@@ -97,6 +105,7 @@ extern int basil_node_ranking(struct node_record *node_array, int node_cnt)
 	hostlist_t hl = hostlist_create(NULL);
 	bool bad_node = 0;
 
+	node_rank_inv = 1;
 	/*
 	 * When obtaining the initial configuration, we can not allow ALPS to
 	 * fail. If there is a problem at this stage it is better to restart
@@ -142,8 +151,47 @@ extern int basil_node_ranking(struct node_record *node_array, int node_cnt)
 			      node->node_id, nam_noderole[node->role],
 			      nam_nodestate[node->state]);
 			bad_node = 1;
-		} else
+		} else if ((slurmctld_conf.fast_schedule != 2)
+			   && (node->cpu_count != node_ptr->config_ptr->cpus)) {
+			fatal("slurm.conf: node %s has %u cpus "
+			      "but configured as CPUs=%u in your slurm.conf",
+			      node_ptr->name, node->cpu_count,
+			      node_ptr->config_ptr->cpus);
+		} else if ((slurmctld_conf.fast_schedule != 2)
+			   && (node->mem_size
+			       != node_ptr->config_ptr->real_memory)) {
+			fatal("slurm.conf: node %s has RealMemory=%u "
+			      "but configured as RealMemory=%u in your "
+			      "slurm.conf",
+			      node_ptr->name, node->mem_size,
+			      node_ptr->config_ptr->real_memory);
+		} else {
 			node_ptr->node_rank = inv->nodes_total - rank_count++;
+			/*
+			 * Convention: since we are using SLURM in
+			 *             frontend-mode, we use
+			 *             NodeHostName as follows.
+			 *
+			 * NodeHostName:  c#-#c#s#n# using the  NID convention
+			 *                <cabinet>-<row><chassis><slot><node>
+			 * - each cabinet can accommodate 3 chassis (c1..c3)
+			 * - each chassis has 8 slots               (s0..s7)
+			 * - each slot contains 2 or 4 nodes        (n0..n3)
+			 *   o either 2 service nodes (n0/n3)
+			 *   o or 4 compute nodes     (n0..n3)
+			 *   o or 2 gemini chips      (g0/g1 serving n0..n3)
+			 *
+			 * Example: c0-0c1s0n1
+			 *          - c0- = cabinet 0
+			 *          - 0   = row     0
+			 *          - c1  = chassis 1
+			 *          - s0  = slot    0
+			 *          - n1  = node    1
+			 */
+			xfree(node_ptr->node_hostname);
+			node_ptr->node_hostname = xstrdup(node->name);
+		}
+
 		sprintf(tmp, "nid%05u", node->node_id);
 		hostlist_push(hl, tmp);
 	}
@@ -156,6 +204,7 @@ extern int basil_node_ranking(struct node_record *node_array, int node_cnt)
 		     "about\n%s", name);
 	}
 	hostlist_destroy(hl);
+	node_rank_inv = 0;
 
 	return SLURM_SUCCESS;
 }
@@ -402,19 +451,12 @@ extern int basil_geometry(struct node_record *node_ptr_array, int node_cnt)
 	 * The processor table has more authoritative information, if a nodeid
 	 * is not listed there, it does not exist.
 	 */
-	const char query[] =	"SELECT x_coord, y_coord, z_coord,"
-				"       cab_position, cab_row, cage, slot, cpu,"
-				"	LOG2(coremask+1), availmem, "
-				"       processor_type  "
-				"FROM  processor LEFT JOIN attributes "
-				"ON    processor_id = nodeid "
-				"WHERE processor_id = ? ";
+	const char query[] =	"SELECT x_coord, y_coord, z_coord, "
+		"processor_type FROM processor WHERE processor_id = ? ";
 	const int	PARAM_COUNT = 1;	/* node id */
 	MYSQL_BIND	params[PARAM_COUNT];
 
 	int		x_coord, y_coord, z_coord;
-	int		cab, row, cage, slot, cpu;
-	unsigned int	node_cpus, node_mem;
 	char		proc_type[BASIL_STRING_SHORT];
 	MYSQL_BIND	bind_cols[COLUMN_COUNT];
 	my_bool		is_null[COLUMN_COUNT];
@@ -439,19 +481,12 @@ extern int basil_geometry(struct node_record *node_ptr_array, int node_cnt)
 			bind_cols[i].buffer	   = proc_type;
 		} else {
 			bind_cols[i].buffer_type   = MYSQL_TYPE_LONG;
-			bind_cols[i].is_unsigned   = (i >= COL_CORES);
+			bind_cols[i].is_unsigned   = (i >= COL_TYPE);
 		}
 	}
 	bind_cols[COL_X].buffer	     = (char *)&x_coord;
 	bind_cols[COL_Y].buffer	     = (char *)&y_coord;
 	bind_cols[COL_Z].buffer	     = (char *)&z_coord;
-	bind_cols[COL_CAB].buffer    = (char *)&cab;
-	bind_cols[COL_ROW].buffer    = (char *)&row;
-	bind_cols[COL_CAGE].buffer   = (char *)&cage;
-	bind_cols[COL_SLOT].buffer   = (char *)&slot;
-	bind_cols[COL_CPU].buffer    = (char *)&cpu;
-	bind_cols[COL_CORES].buffer  = (char *)&node_cpus;
-	bind_cols[COL_MEMORY].buffer = (char *)&node_mem;
 
 	inv = get_full_inventory(version);
 	if (inv == NULL)
@@ -469,7 +504,7 @@ extern int basil_geometry(struct node_record *node_ptr_array, int node_cnt)
 		fatal("can not determine Cray XT/XE system type");
 
 	stmt = prepare_stmt(handle, query, params, PARAM_COUNT,
-				    bind_cols, COLUMN_COUNT);
+			    bind_cols, COLUMN_COUNT);
 	if (stmt == NULL)
 		fatal("can not prepare statement to resolve Cray coordinates");
 
@@ -489,10 +524,8 @@ extern int basil_geometry(struct node_record *node_ptr_array, int node_cnt)
 
 		if (fetch_stmt(stmt) == 0) {
 #if _DEBUG
-			info("proc_type:%s cpus:%u memory:%u",
-			     proc_type, node_cpus, node_mem);
-			info("row:%u cage:%u slot:%u cpu:%u xyz:%u:%u:%u",
-			     row, cage, slot, cpu, x_coord, y_coord, z_coord);
+			info("proc_type:%s xyz:%u:%u:%u",
+			     proc_type, x_coord, y_coord, z_coord
 #endif
 			if (strcmp(proc_type, "compute") != 0) {
 				/*
@@ -502,20 +535,8 @@ extern int basil_geometry(struct node_record *node_ptr_array, int node_cnt)
 				fatal("Node '%s' is a %s node. "
 				      "Only compute nodes can appear in slurm.conf.",
 					node_ptr->name, proc_type);
-			} else if (is_null[COL_CORES] || is_null[COL_MEMORY]) {
-				/*
-				 * This can happen if a node has been disabled
-				 * on the SMW (using 'xtcli disable <nid>'). The
-				 * node will still be listed in the 'processor'
-				 * table, but have no 'attributes' entry (NULL
-				 * values for CPUs/memory). Also, the node will
-				 * be invisible to ALPS, which is why we need to
-				 * set it down here already.
-				 */
-				node_cpus = node_mem = 0;
-				reason = "node data unknown - disabled on SMW?";
 			} else if (is_null[COL_X] || is_null[COL_Y]
-						  || is_null[COL_Z]) {
+				   || is_null[COL_Z]) {
 				/*
 				 * Similar case to the one above, observed when
 				 * a blade has been removed. Node will not
@@ -523,24 +544,6 @@ extern int basil_geometry(struct node_record *node_ptr_array, int node_cnt)
 				 */
 				x_coord = y_coord = z_coord = 0;
 				reason = "unknown coordinates - hardware failure?";
-			} else if (node_cpus < node_ptr->config_ptr->cpus) {
-				/*
-				 * FIXME: Might reconsider this policy.
-				 *
-				 * FastSchedule is ignored here, it requires the
-				 * slurm.conf to be consistent with hardware.
-				 *
-				 * Assumption is that CPU/Memory do not change
-				 * at runtime (Cray has no hot-swappable parts).
-				 *
-				 * Hence checking it in basil_inventory() would
-				 * mean a lot of runtime overhead.
-				 */
-				fatal("slurm.conf: node %s has only Procs=%d",
-					node_ptr->name, node_cpus);
-			} else if (node_mem < node_ptr->config_ptr->real_memory) {
-				fatal("slurm.conf: node %s has RealMemory=%d",
-					node_ptr->name, node_mem);
 			}
 
 		} else if (is_gemini) {
@@ -567,32 +570,13 @@ extern int basil_geometry(struct node_record *node_ptr_array, int node_cnt)
 				node_ptr->arch = xstrdup("XE");
 		}
 
-		xfree(node_ptr->node_hostname);
-		xfree(node_ptr->comm_name);
 		/*
 		 * Convention: since we are using SLURM in frontend-mode,
-		 *             we use Node{Addr,HostName} as follows.
+		 *             we use NodeAddr as follows.
 		 *
 		 * NodeAddr:      <X><Y><Z> coordinates in base-36 encoding
-		 *
-		 * NodeHostName:  c#-#c#s#n# using the  NID convention
-		 *                <cabinet>-<row><chassis><slot><node>
-		 * - each cabinet can accommodate 3 chassis (c1..c3)
-		 * - each chassis has 8 slots               (s0..s7)
-		 * - each slot contains 2 or 4 nodes        (n0..n3)
-		 *   o either 2 service nodes (n0/n3)
-		 *   o or 4 compute nodes     (n0..n3)
-		 *   o or 2 gemini chips      (g0/g1 serving n0..n3)
-		 *
-		 * Example: c0-0c1s0n1
-		 *          - c0- = cabinet 0
-		 *          - 0   = row     0
-		 *          - c1  = chassis 1
-		 *          - s0  = slot    0
-		 *          - n1  = node    1
 		 */
-		node_ptr->node_hostname = xstrdup_printf("c%u-%uc%us%un%u", cab,
-							 row, cage, slot, cpu);
+		xfree(node_ptr->comm_name);
 		node_ptr->comm_name = xstrdup_printf("%c%c%c",
 						     _enc_coord(x_coord),
 						     _enc_coord(y_coord),
@@ -601,9 +585,9 @@ extern int basil_geometry(struct node_record *node_ptr_array, int node_cnt)
 		dim_size[1] = MAX(dim_size[1], (y_coord - 1));
 		dim_size[2] = MAX(dim_size[2], (z_coord - 1));
 #if _DEBUG
-		info("%s  %s  %s  cpus=%u, mem=%u reason=%s", node_ptr->name,
+		info("%s  %s  %s reason=%s", node_ptr->name,
 		     node_ptr->node_hostname, node_ptr->comm_name,
-		     node_cpus, node_mem, reason);
+		     reason);
 #endif
 		/*
 		 * Check the current state reported by ALPS inventory, unless it
@@ -728,11 +712,10 @@ struct basil_accel_param* build_accel_param(struct job_record* job_ptr)
 extern int do_basil_reserve(struct job_record *job_ptr)
 {
 	struct nodespec *ns_head = NULL;
-	uint16_t mppwidth = 0, mppdepth, mppnppn;
 	/* mppmem must be at least 1 for gang scheduling to work so
 	 * if you are wondering why gang scheduling isn't working you
 	 * should check your slurm.conf for DefMemPerNode */
-	uint32_t mppmem = 0, node_min_mem = 0;
+	uint32_t mppdepth, mppnppn, mppwidth = 0, mppmem = 0, node_min_mem = 0;
 	uint32_t resv_id;
 	int i, first_bit, last_bit;
 	long rc;
@@ -759,7 +742,15 @@ extern int do_basil_reserve(struct job_record *job_ptr)
 		return SLURM_SUCCESS;		/* no nodes allocated */
 
 	mppdepth = MAX(1, job_ptr->details->cpus_per_task);
-	mppnppn  = job_ptr->details->ntasks_per_node;
+	if (job_ptr->details->ntasks_per_node) {
+		mppnppn  = job_ptr->details->ntasks_per_node;
+	} else if (job_ptr->details->num_tasks) {
+		mppnppn = (job_ptr->details->num_tasks +
+			   job_ptr->job_resrcs->nhosts - 1) /
+			  job_ptr->job_resrcs->nhosts;
+	} else {
+		mppnppn = 1;
+	}
 
 	/* mppmem */
 	if (job_ptr->details->pn_min_memory & MEM_PER_CPU) {
@@ -802,6 +793,12 @@ extern int do_basil_reserve(struct job_record *job_ptr)
 				node_cpus = node_ptr->cpus;
 				node_mem  = node_ptr->real_memory;
 			}
+
+			/* If the job has requested memory use it (if
+			   lesser) for calculations.
+			*/
+			tmp_mppmem = MIN(node_mem, node_min_mem);
+
 			/*
 			 * ALPS 'Processing Elements per Node' value (aprun -N),
 			 * which in slurm is --ntasks-per-node and 'mppnppn' in
@@ -812,13 +809,10 @@ extern int do_basil_reserve(struct job_record *job_ptr)
 			 * mppmem and use it as the level for all
 			 * nodes (mppmem is 0 when coming in).
 			 */
-			node_mem /= mppnppn ? mppnppn : node_cpus;
-			tmp_mppmem = node_min_mem = MIN(node_mem, node_min_mem);
+			tmp_mppmem /= mppnppn ? mppnppn : node_cpus;
 
-			/* If less than or equal to 0 make sure you
-			   have 1 at least since 0 means give all the
-			   memory to the job.
-			*/
+			/* Minimum memory per processing element should be 1,
+			 * since 0 means give all the memory to the job. */
 			if (tmp_mppmem <= 0)
 				tmp_mppmem = 1;
 
@@ -831,7 +825,7 @@ extern int do_basil_reserve(struct job_record *job_ptr)
 
 	/* mppwidth */
 	for (i = 0; i < job_ptr->job_resrcs->nhosts; i++) {
-		uint16_t node_tasks = job_ptr->job_resrcs->cpus[i] / mppdepth;
+		uint32_t node_tasks = job_ptr->job_resrcs->cpus[i] / mppdepth;
 
 		if (mppnppn && mppnppn < node_tasks)
 			node_tasks = mppnppn;
@@ -883,9 +877,20 @@ extern int do_basil_reserve(struct job_record *job_ptr)
  */
 extern int do_basil_confirm(struct job_record *job_ptr)
 {
+	uint8_t  confirmed;
 	uint32_t resv_id;
 	uint64_t pagg_id;
 
+	if (_get_select_jobinfo(job_ptr->select_jobinfo->data,
+			SELECT_JOBDATA_CONFIRMED, &confirmed)
+			!= SLURM_SUCCESS) {
+		error("can not read confirmed for JobId=%u", job_ptr->job_id);
+	} else if (confirmed != 0) {
+		debug2("ALPS reservation for JobId %u previously confirmed",
+		       job_ptr->job_id);
+		return SLURM_SUCCESS;
+	}
+
 	if (_get_select_jobinfo(job_ptr->select_jobinfo->data,
 			SELECT_JOBDATA_RESV_ID, &resv_id) != SLURM_SUCCESS) {
 		error("can not read resId for JobId=%u", job_ptr->job_id);
@@ -911,6 +916,9 @@ extern int do_basil_confirm(struct job_record *job_ptr)
 		if (rc == 0) {
 			debug2("confirmed ALPS resId %u for JobId %u, pagg "
 			       "%"PRIu64"", resv_id, job_ptr->job_id, pagg_id);
+			confirmed = 1;
+			_set_select_jobinfo(job_ptr->select_jobinfo->data,
+				SELECT_JOBDATA_CONFIRMED, &confirmed);
 			return SLURM_SUCCESS;
 		} else if (rc == -BE_NO_RESID) {
 			/*
@@ -924,12 +932,13 @@ extern int do_basil_confirm(struct job_record *job_ptr)
 			error("JobId %u has invalid ALPS resId %u - job "
 			      "already canceled?", job_ptr->job_id, resv_id);
 			return SLURM_SUCCESS;
+		} else if (is_transient_error(rc)) {
+			debug("confirming ALPS resId %u of JobId %u FAILED: %s",
+			      resv_id, job_ptr->job_id, basil_strerror(rc));
+			return READY_JOB_ERROR;
 		} else {
 			error("confirming ALPS resId %u of JobId %u FAILED: %s",
-				resv_id, job_ptr->job_id, basil_strerror(rc));
-
-			if (is_transient_error(rc))
-				return READY_JOB_ERROR;
+			      resv_id, job_ptr->job_id, basil_strerror(rc));
 		}
 	}
 	return READY_JOB_FATAL;
@@ -1011,6 +1020,7 @@ extern void queue_basil_signal(struct job_record *job_ptr, int signal,
 	if (pthread_attr_setdetachstate(&attr_sig_basil,
 					PTHREAD_CREATE_DETACHED)) {
 		error("pthread_attr_setdetachstate error %m");
+		slurm_attr_destroy(&attr_sig_basil);
 		return;
 	}
 	args_sig_basil = xmalloc(sizeof(args_sig_basil_t));
@@ -1020,6 +1030,8 @@ extern void queue_basil_signal(struct job_record *job_ptr, int signal,
 	if (pthread_create(&thread_sig_basil, &attr_sig_basil,
 			_sig_basil, (void *) args_sig_basil)) {
 		error("pthread_create error %m");
+		slurm_attr_destroy(&attr_sig_basil);
+		xfree(args_sig_basil);
 		return;
 	}
 	slurm_attr_destroy(&attr_sig_basil);
