@@ -179,6 +179,27 @@ void print_fields(type_t type, void *object)
 					     tmp_char,
 					     (curr_inx == field_count));
 			break;
+		case PRINT_ACT_CPUFREQ:
+			switch (type) {
+			case JOB:
+				if (!job->track_steps)
+					tmp_dub = step->stats.act_cpufreq;
+				break;
+			case JOBSTEP:
+				tmp_dub = step->stats.act_cpufreq;
+				break;
+			default:
+				break;
+			}
+			if (!fuzzy_equal(tmp_dub, NO_VAL))
+				convert_num_unit((float)tmp_dub,
+						outbuf, sizeof(outbuf),
+						UNIT_NONE);
+
+			field->print_routine(field,
+					     outbuf,
+					     (curr_inx == field_count));
+			break;
 		case PRINT_ASSOCID:
 			switch(type) {
 			case JOB:
@@ -209,6 +230,7 @@ void print_fields(type_t type, void *object)
 			default:
 				break;
 			}
+
 			if (!fuzzy_equal(tmp_dub, NO_VAL))
 				tmp_char = _elapsed_time((long)tmp_dub, 0);
 
@@ -331,6 +353,27 @@ void print_fields(type_t type, void *object)
 					     tmp_char,
 					     (curr_inx == field_count));
 			break;
+		case PRINT_CONSUMED_ENERGY:
+			switch (type) {
+			case JOB:
+				if (!job->track_steps)
+					tmp_dub = step->stats.consumed_energy;
+				break;
+			case JOBSTEP:
+				tmp_dub = step->stats.consumed_energy;
+				break;
+			default:
+				break;
+			}
+			if (!fuzzy_equal(tmp_dub, NO_VAL))
+				convert_num_unit((float)tmp_dub,
+						 outbuf, sizeof(outbuf),
+						 UNIT_NONE);
+
+			field->print_routine(field,
+					     outbuf,
+					     (curr_inx == field_count));
+			break;
 		case PRINT_CPU_TIME:
 			switch(type) {
 			case JOB:
@@ -646,6 +689,8 @@ void print_fields(type_t type, void *object)
 				tmp_int = NO_VAL;
 				break;
 			}
+			if (tmp_int == (uint16_t)NO_VAL)
+				tmp_int = NO_VAL;
 			field->print_routine(field,
 					     tmp_int,
 					     (curr_inx == field_count));
@@ -709,6 +754,8 @@ void print_fields(type_t type, void *object)
 				tmp_int = NO_VAL;
 				break;
 			}
+			if (tmp_int == (uint16_t)NO_VAL)
+				tmp_int = NO_VAL;
 			field->print_routine(field,
 					     tmp_int,
 					     (curr_inx == field_count));
@@ -774,6 +821,8 @@ void print_fields(type_t type, void *object)
 				tmp_int = NO_VAL;
 				break;
 			}
+			if (tmp_int == (uint16_t)NO_VAL)
+				tmp_int = NO_VAL;
 			field->print_routine(field,
 					     tmp_int,
 					     (curr_inx == field_count));
@@ -835,6 +884,8 @@ void print_fields(type_t type, void *object)
 				tmp_int = NO_VAL;
 				break;
 			}
+			if (tmp_int == (uint16_t)NO_VAL)
+				tmp_int = NO_VAL;
 			field->print_routine(field,
 					     tmp_int,
 					     (curr_inx == field_count));
