@@ -626,6 +626,7 @@ extern int sacctmgr_add_qos(int argc, char *argv[])
 			else
 				qos->description = xstrdup(name);
 
+			qos->flags = start_qos->flags;
 			qos->grace_time = start_qos->grace_time;
 			qos->grp_cpu_mins = start_qos->grp_cpu_mins;
 			qos->grp_cpus = start_qos->grp_cpus;
@@ -662,6 +663,11 @@ extern int sacctmgr_add_qos(int argc, char *argv[])
 	list_iterator_destroy(itr);
 	list_destroy(name_list);
 
+	if (g_qos_list) {
+		list_destroy(g_qos_list);
+		g_qos_list = NULL;
+	}
+
 	if(!list_count(qos_list)) {
 		printf(" Nothing new added.\n");
 		goto end_it;
