@@ -1065,6 +1065,7 @@ extern int sacctmgr_add_user(int argc, char *argv[])
 						start_assoc.grp_cpu_mins;
 					assoc->grp_cpus = start_assoc.grp_cpus;
 					assoc->grp_jobs = start_assoc.grp_jobs;
+					assoc->grp_mem = start_assoc.grp_mem;
 					assoc->grp_nodes =
 						start_assoc.grp_nodes;
 					assoc->grp_submit_jobs =
@@ -1133,6 +1134,7 @@ extern int sacctmgr_add_user(int argc, char *argv[])
 					start_assoc.grp_cpu_mins;
 				assoc->grp_cpus = start_assoc.grp_cpus;
 				assoc->grp_jobs = start_assoc.grp_jobs;
+				assoc->grp_mem = start_assoc.grp_mem;
 				assoc->grp_nodes = start_assoc.grp_nodes;
 				assoc->grp_submit_jobs =
 					start_assoc.grp_submit_jobs;
@@ -1553,7 +1555,7 @@ extern int sacctmgr_list_user(int argc, char *argv[])
 									name);
 							break;
 						}
-						
+
 						/* This means there wasn't a
 						   default on the
 						   current cluster.
@@ -1931,13 +1933,14 @@ extern int sacctmgr_delete_user(int argc, char *argv[])
 	}
 
 	notice_thread_init();
-	if(cond_set == 1) {
+	if (cond_set == 1) {
 		ret_list = acct_storage_g_remove_users(
 			db_conn, my_uid, user_cond);
-	} else if(cond_set & 2) {
+	} else if (cond_set & 2) {
 		ret_list = acct_storage_g_remove_associations(
 			db_conn, my_uid, user_cond->assoc_cond);
 	}
+
 	rc = errno;
 	notice_thread_fini();
 
@@ -1945,6 +1948,7 @@ extern int sacctmgr_delete_user(int argc, char *argv[])
 
 	if(ret_list && list_count(ret_list)) {
 		char *object = NULL;
+		List del_user_list = NULL;
 		ListIterator itr = list_iterator_create(ret_list);
 		/* If there were jobs running with an association to
 		   be deleted, don't.
@@ -1967,8 +1971,82 @@ extern int sacctmgr_delete_user(int argc, char *argv[])
 		}
 		while((object = list_next(itr))) {
 			printf("  %s\n", object);
+			if (cond_set & 2) {
+				if (!del_user_list)
+					del_user_list = list_create(
+						slurm_destroy_char);
+				slurm_addto_char_list(del_user_list,
+						      strstr(object, "U = ")+4);
+			}
 		}
 		list_iterator_destroy(itr);
+
+		/* Remove user if no associations left. */
+		if (cond_set & 2 && del_user_list) {
+			List user_list = NULL;
+			slurmdb_user_cond_t del_user_cond;
+			slurmdb_association_cond_t del_user_assoc_cond;
+			slurmdb_user_rec_t *user = NULL;
+
+			/* Use a fresh cond here so we check all
+			   clusters and such to make sure there are no
+			   associations.
+			*/
+			memset(&del_user_cond, 0, sizeof(slurmdb_user_cond_t));
+			memset(&del_user_assoc_cond, 0,
+			       sizeof(slurmdb_association_cond_t));
+			del_user_cond.with_assocs = 1;
+			del_user_assoc_cond.user_list = del_user_list;
+			/* No need to get all the extra info about the
+			   association, just want to know if it
+			   exists.
+			*/
+			del_user_assoc_cond.without_parent_info = 1;
+			del_user_cond.assoc_cond = &del_user_assoc_cond;
+			user_list = acct_storage_g_get_users(
+				db_conn, my_uid, &del_user_cond);
+			list_destroy(del_user_list);
+			del_user_list = NULL;
+
+			if (user_list) {
+				itr = list_iterator_create(user_list);
+				while ((user = list_next(itr))) {
+					if (user->assoc_list)
+						continue;
+					if (!del_user_list) {
+						del_user_list = list_create(
+							slurm_destroy_char);
+						printf(" Deleting users "
+						       "(No Associations)"
+						       "...\n");
+					}
+					printf("  %s\n", user->name);
+					slurm_addto_char_list(del_user_list,
+							      user->name);
+				}
+				list_iterator_destroy(itr);
+				list_destroy(user_list);
+			}
+
+			if (del_user_list) {
+				List del_user_ret_list = NULL;
+
+				memset(&del_user_cond, 0,
+				       sizeof(slurmdb_user_cond_t));
+				memset(&del_user_assoc_cond, 0,
+				       sizeof(slurmdb_association_cond_t));
+
+				del_user_assoc_cond.user_list = del_user_list;
+				del_user_cond.assoc_cond = &del_user_assoc_cond;
+
+				del_user_ret_list = acct_storage_g_remove_users(
+					db_conn, my_uid, &del_user_cond);
+				if (del_user_ret_list)
+					list_destroy(del_user_ret_list);
+				list_destroy(del_user_list);
+			}
+		}
+
 		if(commit_check("Would you like to commit changes?")) {
 			acct_storage_g_commit(db_conn, 1);
 		} else {
@@ -1984,8 +2062,7 @@ extern int sacctmgr_delete_user(int argc, char *argv[])
 		rc = SLURM_ERROR;
 	}
 
-
-	if(ret_list)
+	if (ret_list)
 		list_destroy(ret_list);
 
 	return rc;
@@ -2067,7 +2144,7 @@ extern int sacctmgr_delete_coord(int argc, char *argv[])
 		printf(" Removing all users from Accounts\n%s", acct_str);
 
 	notice_thread_init();
-        ret_list = acct_storage_g_remove_coord(db_conn, my_uid,
+	ret_list = acct_storage_g_remove_coord(db_conn, my_uid,
 					       user_cond->assoc_cond->acct_list,
 					       user_cond);
 	slurmdb_destroy_user_cond(user_cond);
