@@ -117,7 +117,7 @@ extern int	scontrol_checkpoint(char *op, char *job_step_id_str, int argc,
 				    char **argv);
 extern int	scontrol_create_part(int argc, char *argv[]);
 extern int	scontrol_create_res(int argc, char *argv[]);
-extern int	scontrol_encode_hostlist(char *hostlist);
+extern int	scontrol_encode_hostlist(char *hostlist, bool sorted);
 extern uint16_t	scontrol_get_job_state(uint32_t job_id);
 extern int	scontrol_hold(char *op, char *job_id_str);
 extern int	scontrol_job_notify(int argc, char *argv[]);
