@@ -76,7 +76,7 @@ static bool  _node_state_equal (int state_id, const char *state_str);
 static int   _node_state_id (char *str);
 static const char * _node_state_list (void);
 static void  _parse_token( char *token, char *field, int *field_size,
-                           bool *right_justify, char **suffix);
+			   bool *right_justify, char **suffix);
 static void  _print_options( void );
 static void  _usage( void );
 
@@ -108,6 +108,7 @@ extern void parse_command_line(int argc, char *argv[])
 		{"summarize", no_argument,       0, 's'},
 		{"sort",      required_argument, 0, 'S'},
 		{"states",    required_argument, 0, 't'},
+		{"reservation",no_argument,      0, 'T'},
 		{"verbose",   no_argument,       0, 'v'},
 		{"version",   no_argument,       0, 'V'},
 		{"help",      no_argument,       0, OPT_LONG_HELP},
@@ -126,17 +127,13 @@ extern void parse_command_line(int argc, char *argv[])
 		params.sort = xstrdup(env_val);
 	if ( ( env_val = getenv("SLURM_CLUSTERS") ) ) {
 		if (!(params.clusters = slurmdb_get_info_cluster(env_val))) {
-			error("'%s' can't be reached now, "
-			      "or it is an invalid entry for "
-			      "SLURM_CLUSTERS.  Use 'sacctmgr --list "
-			      "cluster' to see available clusters.",
-			      env_val);
+			print_db_notok(env_val, 1);
 			exit(1);
 		}
 		working_cluster_rec = list_peek(params.clusters);
 	}
 
-	while((opt_char = getopt_long(argc, argv, "abdehi:lM:n:No:p:rRsS:t:vV",
+	while((opt_char = getopt_long(argc, argv, "abdehi:lM:n:No:p:rRsS:t:TvV",
 			long_options, &option_index)) != -1) {
 		switch (opt_char) {
 		case (int)'?':
@@ -184,11 +181,7 @@ extern void parse_command_line(int argc, char *argv[])
 				list_destroy(params.clusters);
 			if (!(params.clusters =
 			      slurmdb_get_info_cluster(optarg))) {
-				error("'%s' can't be reached now, "
-				      "or it is an invalid entry for "
-				      "--cluster.  Use 'sacctmgr --list "
-				      "cluster' to see available clusters.",
-				      optarg);
+				print_db_notok(optarg, 0);
 				exit(1);
 			}
 			working_cluster_rec = list_peek(params.clusters);
@@ -240,6 +233,9 @@ extern void parse_command_line(int argc, char *argv[])
 				exit (1);
 			}
 			break;
+		case (int) 'T':
+			params.reservation_flag = true;
+			break;
 		case (int) 'v':
 			params.verbose++;
 			break;
@@ -262,12 +258,14 @@ extern void parse_command_line(int argc, char *argv[])
 
 	if ( params.format == NULL ) {
 		if ( params.summarize ) {
-			if(params.cluster_flags & CLUSTER_FLAG_BG)
+			params.part_field_flag = true;	/* compute size later */
+			if (params.cluster_flags & CLUSTER_FLAG_BG)
 				params.format = "%9P %.5a %.10l %.32F  %N";
 			else
 				params.format = "%9P %.5a %.10l %.16F  %N";
 		} else if ( params.node_flag ) {
 			params.node_field_flag = true;	/* compute size later */
+			params.part_field_flag = true;	/* compute size later */
 			params.format = params.long_output ?
 			  "%N %.6D %.9P %.11T %.4c %.8z %.6m %.8d %.6w %.8f %20E" :
 			  "%N %.6D %.9P %6t";
@@ -281,6 +279,7 @@ extern void parse_command_line(int argc, char *argv[])
 			params.format = xstrdup(env_val);
 
 		} else {
+			params.part_field_flag = true;	/* compute size later */
 			params.format = params.long_output ?
 			  "%9P %.5a %.10l %.10s %.4r %.5h %.10g %.6D %.11T %N" :
 			  "%9P %.5a %.10l %.6D %.6t %N";
@@ -307,25 +306,25 @@ _next_tok (char *sep, char **str)
 {
 	char *tok;
 
-        /* push str past any leading separators */
-        while ((**str != '\0') && (strchr(sep, **str) != '\0'))
-                (*str)++;
+	/* push str past any leading separators */
+	while ((**str != '\0') && (strchr(sep, **str) != '\0'))
+		(*str)++;
 
-        if (**str == '\0')
-                return NULL;
+	if (**str == '\0')
+		return NULL;
 
-        /* assign token ptr */
-        tok = *str;
+	/* assign token ptr */
+	tok = *str;
 
-        /* push str past token and leave pointing to first separator */
-        while ((**str != '\0') && (strchr(sep, **str) == '\0'))
-                (*str)++;
+	/* push str past token and leave pointing to first separator */
+	while ((**str != '\0') && (strchr(sep, **str) == '\0'))
+		(*str)++;
 
-        /* nullify consecutive separators and push str beyond them */
-        while ((**str != '\0') && (strchr(sep, **str) != '\0'))
-                *(*str)++ = '\0';
+	/* nullify consecutive separators and push str beyond them */
+	while ((**str != '\0') && (strchr(sep, **str) != '\0'))
+		*(*str)++ = '\0';
 
-        return (tok);
+	return (tok);
 }
 
 /*
@@ -615,6 +614,12 @@ _parse_format( char* format )
 					field_size,
 					right_justify,
 					suffix );
+		} else if (field[0] == 'O') {
+			params.match_flags.cpu_load_flag = true;
+			format_add_cpu_load( params.format_list,
+					field_size,
+					right_justify,
+					suffix );
 		} else if (field[0] == 'p') {
 			params.match_flags.priority_flag = true;
 			format_add_priority( params.format_list,
@@ -708,13 +713,13 @@ _parse_format( char* format )
 			prefix = xstrdup("%");
 			xstrcat(prefix, token);
 			xfree(suffix);
-			suffix = prefix;			
+			suffix = prefix;
 			format_add_invalid( params.format_list,
 					    field_size,
 					    right_justify,
 					    suffix );
 			fprintf(stderr, "Invalid node format specification: %c\n",
-			        field[0] );
+				field[0] );
 		}
 		token = strtok_r( NULL, "%", &tmp_char);
 	}
@@ -793,6 +798,8 @@ void _print_options( void )
 					"true" : "false");
 	printf("node_format = %s\n", params.node_flag   ? "true" : "false");
 	printf("nodes       = %s\n", params.nodes ? params.nodes : "n/a");
+	printf("part_field  = %s\n", params.part_field_flag ?
+					"true" : "false");
 	printf("partition   = %s\n", params.partition ?
 					params.partition: "n/a");
 	printf("responding  = %s\n", params.responding_nodes ?
@@ -834,6 +841,8 @@ void _print_options( void )
 			params.match_flags.reason_timestamp_flag ?  "true" : "false");
 	printf("reason_user_flag = %s\n",
 			params.match_flags.reason_user_flag ?  "true" : "false");
+	printf("reservation_flag = %s\n", params.reservation_flag ?
+			"true" : "false");
 	printf("root_flag       = %s\n", params.match_flags.root_flag ?
 			"true" : "false");
 	printf("share_flag      = %s\n", params.match_flags.share_flag ?
@@ -848,7 +857,7 @@ void _print_options( void )
 static void _usage( void )
 {
 	printf("\
-Usage: sinfo [-abdelNRrsv] [-i seconds] [-t states] [-p partition] [-n nodes]\n\
+Usage: sinfo [-abdelNRrsTv] [-i seconds] [-t states] [-p partition] [-n nodes]\n\
              [-S fields] [-o format] \n");
 }
 
@@ -874,6 +883,7 @@ Usage: sinfo [OPTIONS]\n\
   -s, --summarize            report state summary only\n\
   -S, --sort=fields          comma separated list of fields to sort on\n\
   -t, --states=node_state    specify the what states of nodes to view\n\
+  -T, --reservation          show only reservation information\n\
   -v, --verbose              verbosity level\n\
   -V, --version              output version information and exit\n\
 \nHelp options:\n\
