@@ -55,15 +55,22 @@
 #include "src/sinfo/sinfo.h"
 
 #define MIN_NODE_FIELD_SIZE 9
+#define MIN_PART_FIELD_SIZE 9
 
 static int   _build_min_max_16_string(char *buffer, int buf_size,
 				uint16_t min, uint16_t max, bool range);
 static int   _build_min_max_32_string(char *buffer, int buf_size,
 				uint32_t min, uint32_t max,
 				bool range, bool use_suffix);
+static int   _build_cpu_load_min_max_32(char *buffer, int buf_size,
+					uint32_t min, uint32_t max,
+					bool range);
+static void  _print_reservation(reserve_info_t *resv_ptr, int width);
 static int   _print_secs(long time, int width, bool right, bool cut_output);
 static int   _print_str(char *str, int width, bool right, bool cut_output);
+static int   _resv_name_width(reserve_info_t *resv_ptr);
 static void  _set_node_field_size(List sinfo_list);
+static void  _set_part_field_size(List sinfo_list);
 static char *_str_tolower(char *upper_str);
 
 /*****************************************************************************
@@ -84,6 +91,8 @@ int print_sinfo_list(List sinfo_list)
 
 	if (params.node_field_flag)
 		_set_node_field_size(sinfo_list);
+	if (params.part_field_flag)
+		_set_part_field_size(sinfo_list);
 
 	if (!params.no_header)
 		print_sinfo_entry(NULL);
@@ -112,9 +121,57 @@ int print_sinfo_entry(sinfo_data_t *sinfo_data)
 	return SLURM_SUCCESS;
 }
 
+void print_sinfo_reservation(reserve_info_msg_t *resv_ptr)
+{
+	reserve_info_t *reserve_ptr = NULL;
+	char format[64];
+	int i, width = 9;
+
+	reserve_ptr = resv_ptr->reservation_array;
+	if (!params.no_header) {
+		for (i = 0; i < resv_ptr->record_count; i++)
+			width = MAX(width, _resv_name_width(&reserve_ptr[i]));
+		snprintf(format, sizeof(format),
+			 "%%-%ds  %%8s  %%19s  %%19s  %%11s  %%s\n", width);
+		printf(format,
+		       "RESV_NAME", "STATE", "START_TIME", "END_TIME",
+		       "DURATION", "NODELIST");
+	}
+	for (i = 0; i < resv_ptr->record_count; i++)
+		_print_reservation(&reserve_ptr[i], width);
+}
+
 /*****************************************************************************
  * Local Print Functions
  *****************************************************************************/
+static int _resv_name_width(reserve_info_t *resv_ptr)
+{
+	if (!resv_ptr->name)
+		return 0;
+	return strlen(resv_ptr->name);
+}
+
+static void _print_reservation(reserve_info_t *resv_ptr, int width)
+{
+	char format[64], tmp1[32], tmp2[32], tmp3[32];
+	char *state = "INACTIVE";
+	uint32_t duration;
+	time_t now = time(NULL);
+
+	slurm_make_time_str(&resv_ptr->start_time, tmp1, sizeof(tmp1));
+	slurm_make_time_str(&resv_ptr->end_time,   tmp2, sizeof(tmp2));
+	duration = difftime(resv_ptr->end_time, resv_ptr->start_time);
+	secs2time_str(duration, tmp3, sizeof(tmp3));
+
+	if ((resv_ptr->start_time <= now) && (resv_ptr->end_time >= now))
+		state = "ACTIVE";
+	snprintf(format, sizeof(format),
+		 "%%-%ds  %%8s  %%19s  %%19s  %%11s  %%s\n", width);
+	printf(format,
+	       resv_ptr->name, state, tmp1, tmp2, tmp3, resv_ptr->node_list);
+
+	return;
+}
 
 static int _print_str(char *str, int width, bool right, bool cut_output)
 {
@@ -160,15 +217,15 @@ static int _print_secs(long time, int width, bool right, bool cut_output)
 	if (days)
 		snprintf(str, FORMAT_STRING_SIZE,
 			 "%ld-%2.2ld:%2.2ld:%2.2ld",
-		         days, hours, minutes, seconds);
+			 days, hours, minutes, seconds);
 	else if (hours)
 		snprintf(str, FORMAT_STRING_SIZE,
 			 "%ld:%2.2ld:%2.2ld",
-		         hours, minutes, seconds);
+			 hours, minutes, seconds);
 	else
 		snprintf(str, FORMAT_STRING_SIZE,
 			 "%ld:%2.2ld",
-		         minutes, seconds);
+			 minutes, seconds);
 
 	_print_str(str, width, right, cut_output);
 	return SLURM_SUCCESS;
@@ -225,6 +282,37 @@ _build_min_max_32_string(char *buffer, int buf_size,
 					tmp_min, tmp_max);
 	} else
 		return snprintf(buffer, buf_size, "%s+", tmp_min);
+
+
+}
+
+static int
+_build_cpu_load_min_max_32(char *buffer, int buf_size,
+			    uint32_t min, uint32_t max,
+			    bool range)
+{
+
+	char tmp_min[8];
+	char tmp_max[8];
+
+	if (min == NO_VAL) {
+		strcpy(tmp_min, "N/A");
+	} else {
+		snprintf(tmp_min, sizeof(tmp_min), "%.2f", (min/100.0));
+	}
+
+	if (max == NO_VAL) {
+		strcpy(tmp_max, "N/A");
+	} else {
+		snprintf(tmp_max, sizeof(tmp_max), "%.2f", (max/100.0));
+	}
+
+	if (max == min)
+		return snprintf(buffer, buf_size, "%s", tmp_max);
+	else if (range)
+		return snprintf(buffer, buf_size, "%s-%s", tmp_min, tmp_max);
+	else
+		return snprintf(buffer, buf_size, "%s+", tmp_min);
 }
 
 int
@@ -262,6 +350,22 @@ static void _set_node_field_size(List sinfo_list)
 	params.node_field_size = max_width;
 }
 
+static void _set_part_field_size(List sinfo_list)
+{
+	ListIterator i = list_iterator_create(sinfo_list);
+	sinfo_data_t *current;
+	int max_width = MIN_PART_FIELD_SIZE, this_width = 0;
+
+	while ((current = (sinfo_data_t *) list_next(i)) != NULL) {
+		if (!current->part_info || !current->part_info->name)
+			continue;
+		this_width = strlen(current->part_info->name);
+		max_width = MAX(max_width, this_width);
+	}
+	list_iterator_destroy(i);
+	params.part_field_size = max_width;
+}
+
 /*
  * _str_tolower - convert string to all lower case
  * upper_str IN - upper case input string
@@ -312,8 +416,8 @@ int _print_cpus(sinfo_data_t * sinfo_data, int width,
 	char id[FORMAT_STRING_SIZE];
 	if (sinfo_data) {
 		_build_min_max_32_string(id, FORMAT_STRING_SIZE,
-		                      sinfo_data->min_cpus,
-		                      sinfo_data->max_cpus,
+				      sinfo_data->min_cpus,
+				      sinfo_data->max_cpus,
 				      false, true);
 		_print_str(id, width, right_justify, true);
 	} else
@@ -372,14 +476,14 @@ int _print_sct(sinfo_data_t * sinfo_data, int width,
 	char sct[(FORMAT_STRING_SIZE+1)*3];
 	if (sinfo_data) {
 		_build_min_max_16_string(sockets, FORMAT_STRING_SIZE,
-		                      sinfo_data->min_sockets,
-		                      sinfo_data->max_sockets, false);
+				      sinfo_data->min_sockets,
+				      sinfo_data->max_sockets, false);
 		_build_min_max_16_string(cores, FORMAT_STRING_SIZE,
-		                      sinfo_data->min_cores,
-		                      sinfo_data->max_cores, false);
+				      sinfo_data->min_cores,
+				      sinfo_data->max_cores, false);
 		_build_min_max_16_string(threads, FORMAT_STRING_SIZE,
-		                      sinfo_data->min_threads,
-		                      sinfo_data->max_threads, false);
+				      sinfo_data->min_threads,
+				      sinfo_data->max_threads, false);
 		sct[0] = '\0';
 		strcat(sct, sockets);
 		strcat(sct, ":");
@@ -402,8 +506,8 @@ int _print_sockets(sinfo_data_t * sinfo_data, int width,
 	char id[FORMAT_STRING_SIZE];
 	if (sinfo_data) {
 		_build_min_max_16_string(id, FORMAT_STRING_SIZE,
-		                      sinfo_data->min_sockets,
-		                      sinfo_data->max_sockets, false);
+				      sinfo_data->min_sockets,
+				      sinfo_data->max_sockets, false);
 		_print_str(id, width, right_justify, true);
 	} else {
 		_print_str("SOCKETS", width, right_justify, true);
@@ -420,8 +524,8 @@ int _print_cores(sinfo_data_t * sinfo_data, int width,
 	char id[FORMAT_STRING_SIZE];
 	if (sinfo_data) {
 		_build_min_max_16_string(id, FORMAT_STRING_SIZE,
-		                      sinfo_data->min_cores,
-		                      sinfo_data->max_cores, false);
+				      sinfo_data->min_cores,
+				      sinfo_data->max_cores, false);
 		_print_str(id, width, right_justify, true);
 	} else {
 		_print_str("CORES", width, right_justify, true);
@@ -438,8 +542,8 @@ int _print_threads(sinfo_data_t * sinfo_data, int width,
 	char id[FORMAT_STRING_SIZE];
 	if (sinfo_data) {
 		_build_min_max_16_string(id, FORMAT_STRING_SIZE,
-		                      sinfo_data->min_threads,
-		                      sinfo_data->max_threads, false);
+				      sinfo_data->min_threads,
+				      sinfo_data->max_threads, false);
 		_print_str(id, width, right_justify, true);
 	} else {
 		_print_str("THREADS", width, right_justify, true);
@@ -456,8 +560,8 @@ int _print_disk(sinfo_data_t * sinfo_data, int width,
 	char id[FORMAT_STRING_SIZE];
 	if (sinfo_data) {
 		_build_min_max_32_string(id, FORMAT_STRING_SIZE,
-		                      sinfo_data->min_disk,
-		                      sinfo_data->max_disk,
+				      sinfo_data->min_disk,
+				      sinfo_data->max_disk,
 				      false, false);
 		_print_str(id, width, right_justify, true);
 	} else
@@ -538,8 +642,8 @@ int _print_memory(sinfo_data_t * sinfo_data, int width,
 	char id[FORMAT_STRING_SIZE];
 	if (sinfo_data) {
 		_build_min_max_32_string(id, FORMAT_STRING_SIZE,
-		                      sinfo_data->min_mem,
-		                      sinfo_data->max_mem,
+				      sinfo_data->min_mem,
+				      sinfo_data->max_mem,
 				      false, false);
 		_print_str(id, width, right_justify, true);
 	} else
@@ -657,7 +761,7 @@ int _print_nodes_ai(sinfo_data_t * sinfo_data, int width,
 				 sinfo_data->nodes_idle);
 		}
 		snprintf(id, FORMAT_STRING_SIZE, "%s/%s",
-		         tmpa, tmpi);
+			 tmpa, tmpi);
 		_print_str(id, width, right_justify, true);
 	} else
 		_print_str("NODES(A/I)", width, right_justify, true);
@@ -696,7 +800,7 @@ int _print_nodes_aiot(sinfo_data_t * sinfo_data, int width,
 				 sinfo_data->nodes_total);
 		}
 		snprintf(id, FORMAT_STRING_SIZE, "%s/%s/%s/%s",
-		         tmpa, tmpi, tmpo, tmpt);
+			 tmpa, tmpi, tmpo, tmpt);
 		_print_str(id, width, right_justify, true);
 	} else
 		_print_str("NODES(A/I/O/T)", width, right_justify, true);
@@ -709,6 +813,8 @@ int _print_nodes_aiot(sinfo_data_t * sinfo_data, int width,
 int _print_partition(sinfo_data_t * sinfo_data, int width,
 			bool right_justify, char *suffix)
 {
+	if (params.part_field_flag)
+		width = params.part_field_size;
 	if (sinfo_data) {
 		if (sinfo_data->part_info == NULL)
 			_print_str("n/a", width, right_justify, true);
@@ -735,6 +841,8 @@ int _print_partition(sinfo_data_t * sinfo_data, int width,
 int _print_partition_name(sinfo_data_t * sinfo_data, int width,
 			  bool right_justify, char *suffix)
 {
+	if (params.part_field_flag)
+		width = params.part_field_size;
 	if (sinfo_data) {
 		if (sinfo_data->part_info == NULL)
 			_print_str("n/a", width, right_justify, true);
@@ -765,7 +873,7 @@ int _print_preempt_mode(sinfo_data_t * sinfo_data, int width,
 		uint16_t preempt_mode = sinfo_data->part_info->preempt_mode;
 		if (preempt_mode == (uint16_t) NO_VAL)
 			preempt_mode =  slurm_get_preempt_mode();
-		_print_str(preempt_mode_string(preempt_mode), 
+		_print_str(preempt_mode_string(preempt_mode),
 			   width, right_justify, true);
 	} else
 		_print_str("PREEMPT_MODE", width, right_justify, true);
@@ -782,8 +890,8 @@ int _print_priority(sinfo_data_t * sinfo_data, int width,
 
 	if (sinfo_data) {
 		_build_min_max_16_string(id, FORMAT_STRING_SIZE,
-		                      sinfo_data->part_info->priority,
-		                      sinfo_data->part_info->priority, true);
+				      sinfo_data->part_info->priority,
+				      sinfo_data->part_info->priority, true);
 		_print_str(id, width, right_justify, true);
 	} else
 		_print_str("PRIORITY", width, right_justify, true);
@@ -880,10 +988,18 @@ int _print_size(sinfo_data_t * sinfo_data, int width,
 int _print_state_compact(sinfo_data_t * sinfo_data, int width,
 			bool right_justify, char *suffix)
 {
+	char *upper_state, *lower_state;
+	uint16_t my_state;
+
 	if (sinfo_data && sinfo_data->nodes_total) {
-		char *upper_state = node_state_string_compact(
-			sinfo_data->node_state);
-		char *lower_state = _str_tolower(upper_state);
+		my_state = sinfo_data->node_state;
+		if (sinfo_data->cpus_alloc &&
+		    (sinfo_data->cpus_alloc != sinfo_data->cpus_total)) {
+			my_state &= NODE_STATE_FLAGS;
+			my_state |= NODE_STATE_MIXED;
+		}
+		upper_state = node_state_string_compact(my_state);
+		lower_state = _str_tolower(upper_state);
 		_print_str(lower_state, width, right_justify, true);
 		xfree(lower_state);
 	} else if (sinfo_data)
@@ -899,9 +1015,18 @@ int _print_state_compact(sinfo_data_t * sinfo_data, int width,
 int _print_state_long(sinfo_data_t * sinfo_data, int width,
 			bool right_justify, char *suffix)
 {
+	char *upper_state, *lower_state;
+	uint16_t my_state;
+
 	if (sinfo_data && sinfo_data->nodes_total) {
-		char *upper_state = node_state_string(sinfo_data->node_state);
-		char *lower_state = _str_tolower(upper_state);
+		my_state = sinfo_data->node_state;
+		if (sinfo_data->cpus_alloc &&
+		    (sinfo_data->cpus_alloc != sinfo_data->cpus_total)) {
+			my_state &= NODE_STATE_FLAGS;
+			my_state |= NODE_STATE_MIXED;
+		}
+		upper_state = node_state_string(my_state);
+		lower_state = _str_tolower(upper_state);
 		_print_str(lower_state, width, right_justify, true);
 		xfree(lower_state);
 	} else if (sinfo_data)
@@ -1025,8 +1150,8 @@ int _print_weight(sinfo_data_t * sinfo_data, int width,
 	char id[FORMAT_STRING_SIZE];
 	if (sinfo_data) {
 		_build_min_max_32_string(id, FORMAT_STRING_SIZE,
-		                      sinfo_data->min_weight,
-		                      sinfo_data->max_weight,
+				      sinfo_data->min_weight,
+				      sinfo_data->max_weight,
 				      false, false);
 		_print_str(id, width, right_justify, true);
 	} else
@@ -1044,3 +1169,24 @@ int _print_com_invalid(sinfo_data_t * sinfo_data, int width,
 		printf("%s", suffix);
 	return SLURM_SUCCESS;
 }
+
+int _print_cpu_load(sinfo_data_t * sinfo_data, int width,
+		    bool right_justify, char *suffix)
+{
+	char id[FORMAT_STRING_SIZE];
+
+	if (sinfo_data) {
+		_build_cpu_load_min_max_32(id, FORMAT_STRING_SIZE,
+					 sinfo_data->min_cpu_load,
+					 sinfo_data->max_cpu_load,
+					 true);
+		_print_str(id, width, right_justify, true);
+	} else {
+		_print_str("CPU_LOAD", width, right_justify, true);
+	}
+
+	if (suffix)
+		printf("%s", suffix);
+	return SLURM_SUCCESS;
+
+}
