@@ -178,7 +178,6 @@ static void _purge_agent_args(agent_arg_t *agent_arg_ptr);
 static void _queue_agent_retry(agent_info_t * agent_info_ptr, int count);
 static int _setup_requeue(agent_arg_t *agent_arg_ptr, thd_t *thread_ptr,
 			  int count, int *spot);
-static void _slurmctld_free_batch_job_launch_msg(batch_job_launch_msg_t * msg);
 static void _spawn_retry_agent(agent_arg_t * agent_arg_ptr);
 static void *_thread_per_group_rpc(void *args);
 static int   _valid_agent_arg(agent_arg_t *agent_arg_ptr);
@@ -744,6 +743,7 @@ finished:	;
 	}
 	if ((agent_ptr->msg_type == REQUEST_PING) ||
 	    (agent_ptr->msg_type == REQUEST_HEALTH_CHECK) ||
+	    (agent_ptr->msg_type == REQUEST_ACCT_GATHER_UPDATE) ||
 	    (agent_ptr->msg_type == REQUEST_NODE_REGISTRATION_STATUS))
 		ping_end();
 }
@@ -813,11 +813,15 @@ static void *_thread_per_group_rpc(void *args)
 	/* Lock: Read node */
 	slurmctld_lock_t node_read_lock = {
 		NO_LOCK, NO_LOCK, READ_LOCK, NO_LOCK };
+	/* Lock: Write node */
+	slurmctld_lock_t node_write_lock = {
+		NO_LOCK, NO_LOCK, WRITE_LOCK, NO_LOCK };
 
 	xassert(args != NULL);
 	xsignal(SIGUSR1, _sig_handler);
 	xsignal_unblock(sig_array);
 	is_kill_msg = (	(msg_type == REQUEST_KILL_TIMELIMIT)	||
+			(msg_type == REQUEST_KILL_PREEMPTED)	||
 			(msg_type == REQUEST_TERMINATE_JOB) );
 	srun_agent = (	(msg_type == SRUN_PING)			||
 			(msg_type == SRUN_EXEC)			||
@@ -844,10 +848,10 @@ static void *_thread_per_group_rpc(void *args)
  	info("sending message type %u to %s", msg_type, thread_ptr->nodelist);
 #endif
 	if (task_ptr->get_reply) {
-		if(thread_ptr->addr) {
+		if (thread_ptr->addr) {
 			msg.address = *thread_ptr->addr;
 
-			if(!(ret_list = slurm_send_addr_recv_msgs(
+			if (!(ret_list = slurm_send_addr_recv_msgs(
 				     &msg, thread_ptr->nodelist, 0))) {
 				error("_thread_per_group_rpc: "
 				      "no ret_list given");
@@ -856,7 +860,7 @@ static void *_thread_per_group_rpc(void *args)
 
 
 		} else {
-			if(!(ret_list = slurm_send_recv_msgs(
+			if (!(ret_list = slurm_send_recv_msgs(
 				     thread_ptr->nodelist,
 				     &msg, 0, true))) {
 				error("_thread_per_group_rpc: "
@@ -865,7 +869,7 @@ static void *_thread_per_group_rpc(void *args)
 			}
 		}
 	} else {
-		if(thread_ptr->addr) {
+		if (thread_ptr->addr) {
 			//info("got the address");
 			msg.address = *thread_ptr->addr;
 		} else {
@@ -897,8 +901,17 @@ static void *_thread_per_group_rpc(void *args)
 	while ((ret_data_info = list_next(itr)) != NULL) {
 		rc = slurm_get_return_code(ret_data_info->type,
 					   ret_data_info->data);
-		/* SPECIAL CASE: Mark node as IDLE if job already
-		   complete */
+		/* SPECIAL CASE: Record node's CPU load */
+		if (ret_data_info->type == RESPONSE_PING_SLURMD) {
+			ping_slurmd_resp_msg_t *ping_resp;
+			ping_resp = (ping_slurmd_resp_msg_t *)
+				    ret_data_info->data;
+			lock_slurmctld(node_write_lock);
+			reset_node_load(ret_data_info->node_name,
+					ping_resp->cpu_load);
+			unlock_slurmctld(node_write_lock);
+		}
+		/* SPECIAL CASE: Mark node as IDLE if job already complete */
 		if (is_kill_msg &&
 		    (rc == ESLURMD_KILL_JOB_ALREADY_COMPLETE)) {
 			kill_job_msg_t *kill_job;
@@ -913,6 +926,15 @@ static void *_thread_per_group_rpc(void *args)
 				run_scheduler = true;
 			unlock_slurmctld(job_write_lock);
 		}
+
+		/* SPECIAL CASE: Record node's CPU load */
+		if (ret_data_info->type == RESPONSE_ACCT_GATHER_UPDATE) {
+			lock_slurmctld(node_write_lock);
+			update_node_record_acct_gather_data(
+				ret_data_info->data);
+			unlock_slurmctld(node_write_lock);
+		}
+
 		/* SPECIAL CASE: Kill non-startable batch job,
 		 * Requeue the job on ESLURMD_PROLOG_FAILED */
 		if ((msg_type == REQUEST_BATCH_JOB_LAUNCH) &&
@@ -1328,12 +1350,12 @@ static void _spawn_retry_agent(agent_arg_t * agent_arg_ptr)
 	slurm_attr_destroy(&attr_agent);
 }
 
-/* _slurmctld_free_batch_job_launch_msg is a variant of
+/* slurmctld_free_batch_job_launch_msg is a variant of
  *	slurm_free_job_launch_msg because all environment variables currently
  *	loaded in one xmalloc buffer (see get_job_env()), which is different
  *	from how slurmd assembles the data from a message
  */
-static void _slurmctld_free_batch_job_launch_msg(batch_job_launch_msg_t * msg)
+extern void slurmctld_free_batch_job_launch_msg(batch_job_launch_msg_t * msg)
 {
 	if (msg) {
 		if (msg->environment) {
@@ -1374,8 +1396,8 @@ static void _purge_agent_args(agent_arg_t *agent_arg_ptr)
 	xfree(agent_arg_ptr->addr);
 	if (agent_arg_ptr->msg_args) {
 		if (agent_arg_ptr->msg_type == REQUEST_BATCH_JOB_LAUNCH)
-			_slurmctld_free_batch_job_launch_msg(
-				agent_arg_ptr->msg_args);
+			slurmctld_free_batch_job_launch_msg(agent_arg_ptr->
+							    msg_args);
 		else if (agent_arg_ptr->msg_type ==
 				RESPONSE_RESOURCE_ALLOCATION)
 			slurm_free_resource_allocation_response_msg(
@@ -1399,6 +1421,8 @@ static void _purge_agent_args(agent_arg_t *agent_arg_ptr)
 				agent_arg_ptr->msg_args);
 		else if (agent_arg_ptr->msg_type == REQUEST_JOB_NOTIFY)
 			slurm_free_job_notify_msg(agent_arg_ptr->msg_args);
+		else if (agent_arg_ptr->msg_type == REQUEST_SUSPEND_INT)
+			slurm_free_suspend_int_msg(agent_arg_ptr->msg_args);
 		else
 			xfree(agent_arg_ptr->msg_args);
 	}
