@@ -330,13 +330,16 @@ typedef struct slurmctld_resv {
 	char *accounts;		/* names of accounts permitted to use	*/
 	int account_cnt;	/* count of accounts permitted to use	*/
 	char **account_list;	/* list of accounts permitted to use	*/
+	bool account_not;	/* account_list users NOT permitted to use */
 	char *assoc_list;	/* list of associations			*/
 	uint32_t cpu_cnt;	/* number of reserved CPUs		*/
+	bitstr_t *core_bitmap;	/* bitmap of reserved cores		*/
 	uint32_t duration;	/* time in seconds for this
 				 * reservation to last                  */
 	time_t end_time;	/* end time of reservation		*/
 	char *features;		/* required node features		*/
 	uint16_t flags;		/* see RESERVE_FLAG_* in slurm.h	*/
+	bool full_nodes;	/* when reservation uses full nodes or not */
 	uint32_t job_pend_cnt;	/* number of pending jobs		*/
 	uint32_t job_run_cnt;	/* number of running jobs		*/
 	List license_list;	/* structure with license info		*/
@@ -358,6 +361,7 @@ typedef struct slurmctld_resv {
 	char *users;		/* names of users permitted to use	*/
 	int user_cnt;		/* count of users permitted to use	*/
 	uid_t *user_list;	/* array of users permitted to use	*/
+	bool user_not;		/* user_list users NOT permitted to use	*/
 } slurmctld_resv_t;
 
 /*****************************************************************************\
@@ -501,8 +505,14 @@ struct job_record {
 					 * wait call) */
 	front_end_record_t *front_end_ptr; /* Pointer to front-end node running
 					 * this job */
-	char *gres;			/* generic resources */
+	char *gres;			/* generic resources requested by job*/
 	List gres_list;			/* generic resource allocation detail */
+	char *gres_alloc;		/* Allocated GRES added over all nodes
+					 * to be passed to slurmdbd */
+	char *gres_req;			/* Requested GRES added over all nodes
+					 * to be passed to slurmdbd */
+	char *gres_used;		/* Actual GRES use added over all nodes
+					 * to be passed to slurmdbd */
 	uint32_t group_id;		/* group submitted under */
 	uint32_t job_id;		/* job ID */
 	struct job_record *job_next;	/* next entry with same hash index */
@@ -632,6 +642,7 @@ struct 	step_record {
 					 * step relative to job's nodes,
 					 * see src/common/job_resources.h */
 	uint32_t cpu_count;		/* count of step's CPUs */
+	uint32_t cpu_freq;	        /* requested cpu frequency */
 	uint16_t cpus_per_task;		/* cpus per task initiated */
 	uint16_t cyclic_alloc;		/* set for cyclic task allocation
 					 * across nodes */
@@ -958,7 +969,7 @@ extern int job_alloc_info(uint32_t uid, uint32_t job_id,
  * IN will_run - don't initiate the job if set, just test if it could run
  *	now or later
  * OUT resp - will run response (includes start location, time, etc.)
- * IN allocate - resource allocation request if set, not a full job
+ * IN allocate - resource allocation request only if set, batch job if zero
  * IN submit_uid -uid of user issuing the request
  * OUT job_pptr - set to pointer to job record
  * RET 0 or an error code. If the job would only be able to execute with
@@ -1564,6 +1575,9 @@ extern void reset_first_job_id(void);
  */
 extern void reset_job_bitmaps (void);
 
+/* Reset a node's CPU load value */
+extern void reset_node_load(char *node_name, uint32_t cpu_load);
+
 /* Reset all scheduling statistics
  * level IN - clear backfilled_jobs count if set */
 extern void reset_stats(int level);
@@ -1790,6 +1804,18 @@ extern void update_logging(void);
  */
 extern int update_node ( update_node_msg_t * update_node_msg )  ;
 
+/* Update nodes accounting usage data */
+extern void update_nodes_acct_gather_data(void);
+
+/*
+ * update_node_record_acct_gather_data - update the energy data in the
+ * node_record
+ * IN msg - node energy data message
+ * RET 0 if no error, ENOENT if no such node
+ */
+extern int update_node_record_acct_gather_data(
+	acct_gather_node_resp_msg_t *msg);
+
 /*
  * update_part - create or update a partition's configuration data
  * IN part_desc - description of partition changes
@@ -1822,6 +1848,11 @@ extern int validate_alloc_node(struct part_record *part_ptr, char* alloc_node);
  */
 extern int validate_group (struct part_record *part_ptr, uid_t run_uid);
 
+/* Perform some size checks on strings we store to prevent
+ * malicious user filling slurmctld's memory
+ * RET 0 or error code */
+extern int validate_job_create_req(job_desc_msg_t * job_desc);
+
 /*
  * validate_jobs_on_node - validate that any jobs that should be on the node
  *	are actually running, if not clean up the job records and/or node
