@@ -93,6 +93,7 @@ extern void srun_allocate (uint32_t job_id)
 		msg_arg = xmalloc(sizeof(resource_allocation_response_msg_t));
 		msg_arg->job_id 	= job_ptr->job_id;
 		msg_arg->node_list	= xstrdup(job_ptr->nodes);
+		msg_arg->alias_list	= xstrdup(job_ptr->alias_list);
 		msg_arg->num_cpu_groups	= job_resrcs_ptr->cpu_array_cnt;
 		msg_arg->cpus_per_node  = xmalloc(sizeof(uint16_t) *
 					  job_resrcs_ptr->cpu_array_cnt);
@@ -239,6 +240,31 @@ extern void srun_ping (void)
 	list_iterator_destroy(job_iterator);
 }
 
+/*
+ * srun_step_timeout - notify srun of a job step's imminent timeout
+ * IN step_ptr - pointer to the slurmctld step record
+ * IN timeout_val - when it is going to time out
+ */
+extern void srun_step_timeout(struct step_record *step_ptr, time_t timeout_val)
+{
+	slurm_addr_t *addr;
+	srun_timeout_msg_t *msg_arg;
+
+	xassert(step_ptr);
+
+	if (step_ptr->batch_step || !step_ptr->port
+	    || !step_ptr->host || (step_ptr->host[0] == '\0'))
+		return;
+
+	addr = xmalloc(sizeof(struct sockaddr_in));
+	slurm_set_addr(addr, step_ptr->port, step_ptr->host);
+	msg_arg = xmalloc(sizeof(srun_timeout_msg_t));
+	msg_arg->job_id   = step_ptr->job_ptr->job_id;
+	msg_arg->step_id  = step_ptr->step_id;
+	msg_arg->timeout  = timeout_val;
+	_srun_agent_launch(addr, step_ptr->host, SRUN_TIMEOUT, msg_arg);
+}
+
 /*
  * srun_timeout - notify srun of a job's imminent timeout
  * IN job_ptr - pointer to the slurmctld job record
@@ -267,25 +293,11 @@ extern void srun_timeout (struct job_record *job_ptr)
 
 
 	step_iterator = list_iterator_create(job_ptr->step_list);
-	while ((step_ptr = (struct step_record *) list_next(step_iterator))) {
-		if ( (step_ptr->port    == 0)    ||
-		     (step_ptr->host    == NULL) ||
-		     (step_ptr->batch_step)      ||
-		     (step_ptr->host[0] == '\0') )
-			continue;
-		addr = xmalloc(sizeof(struct sockaddr_in));
-		slurm_set_addr(addr, step_ptr->port, step_ptr->host);
-		msg_arg = xmalloc(sizeof(srun_timeout_msg_t));
-		msg_arg->job_id   = job_ptr->job_id;
-		msg_arg->step_id  = step_ptr->step_id;
-		msg_arg->timeout  = job_ptr->end_time;
-		_srun_agent_launch(addr, step_ptr->host, SRUN_TIMEOUT,
-				   msg_arg);
-	}
+	while ((step_ptr = (struct step_record *) list_next(step_iterator)))
+		srun_step_timeout(step_ptr, job_ptr->end_time);
 	list_iterator_destroy(step_iterator);
 }
 
-
 /*
  * srun_user_message - Send arbitrary message to an srun job (no job steps)
  */
@@ -448,6 +460,30 @@ extern void srun_step_missing (struct step_record *step_ptr,
 	}
 }
 
+/*
+ * srun_step_signal - notify srun that a job step should be signalled
+ * NOTE: Needed on BlueGene/Q to signal runjob process
+ * IN step_ptr  - pointer to the slurmctld job step record
+ * IN signal - signal number
+ */
+extern void srun_step_signal (struct step_record *step_ptr, uint16_t signal)
+{
+	slurm_addr_t * addr;
+	job_step_kill_msg_t *msg_arg;
+
+	xassert(step_ptr);
+	if (step_ptr->port && step_ptr->host && step_ptr->host[0]) {
+		addr = xmalloc(sizeof(struct sockaddr_in));
+		slurm_set_addr(addr, step_ptr->port, step_ptr->host);
+		msg_arg = xmalloc(sizeof(job_step_kill_msg_t));
+		msg_arg->job_id      = step_ptr->job_ptr->job_id;
+		msg_arg->job_step_id = step_ptr->step_id;
+		msg_arg->signal      = signal;
+		_srun_agent_launch(addr, step_ptr->host, SRUN_STEP_SIGNAL,
+				   msg_arg);
+	}
+}
+
 /*
  * srun_exec - request that srun execute a specific command
  *	and route it's output to stdout
