@@ -49,6 +49,9 @@
 #include <sys/types.h> /* for pid_t */
 #include <sys/signal.h> /* for SIGKILL */
 #endif
+#if defined(__FreeBSD__)
+#include <signal.h>
+#endif
 #include <errno.h>
 #include <fcntl.h>
 #include <grp.h>
@@ -74,8 +77,7 @@
 /* Change TRIGGER_STATE_VERSION value when changing the state save format */
 #define TRIGGER_STATE_VERSION      "VER004"
 #define TRIGGER_2_4_STATE_VERSION  "VER004"	/* SLURM version 2.4 */
-#define TRIGGER_2_2_STATE_VERSION  "VER003"	/* SLURM version 2.2 */
-#define TRIGGER_2_1_STATE_VERSION  "VER002"	/* SLURM version 2.1 */
+#define TRIGGER_2_3_STATE_VERSION  "VER003"	/* SLURM version 2.3 */
 
 List trigger_list;
 uint32_t next_trigger_id = 1;
@@ -698,7 +700,7 @@ static int _load_trigger_state(Buf buffer, uint16_t protocol_version)
 		safe_unpack32   (&trig_ptr->group_id,  buffer);
 		safe_unpackstr_xmalloc(&trig_ptr->program, &str_len, buffer);
 		safe_unpack8    (&trig_ptr->state,     buffer);
-	} else if (protocol_version >= SLURM_2_2_PROTOCOL_VERSION) {
+	} else if (protocol_version >= SLURM_2_3_PROTOCOL_VERSION) {
 		/* restore trigger pull state flags */
 		safe_unpack8(&ctld_failure, buffer);
 		safe_unpack8(&bu_ctld_failure, buffer);
@@ -718,20 +720,9 @@ static int _load_trigger_state(Buf buffer, uint16_t protocol_version)
 		safe_unpackstr_xmalloc(&trig_ptr->program, &str_len, buffer);
 		safe_unpack8    (&trig_ptr->state,     buffer);
 	} else {
-		uint16_t uint16_tmp;
-		safe_unpack32   (&trig_ptr->trig_id,   buffer);
-		safe_unpack16   (&trig_ptr->res_type,  buffer);
-		safe_unpackstr_xmalloc(&trig_ptr->res_id, &str_len, buffer);
-		/* rebuild nodes_bitmap as needed from res_id */
-		/* rebuild job_id as needed from res_id */
-		/* rebuild job_ptr as needed from res_id */
-		safe_unpack16   (&uint16_tmp,          buffer);
-		trig_ptr->trig_type = (uint32_t) uint16_tmp;
-		safe_unpack_time(&trig_ptr->trig_time, buffer);
-		safe_unpack32   (&trig_ptr->user_id,   buffer);
-		safe_unpack32   (&trig_ptr->group_id,  buffer);
-		safe_unpackstr_xmalloc(&trig_ptr->program, &str_len, buffer);
-		safe_unpack8    (&trig_ptr->state,     buffer);
+		error("_load_trigger_state: protocol_version "
+		      "%hu not supported", protocol_version);
+		goto unpack_error;
 	}
 
 	if ((trig_ptr->res_type < TRIGGER_RES_TYPE_JOB)  ||
@@ -759,7 +750,7 @@ static int _load_trigger_state(Buf buffer, uint16_t protocol_version)
 	if (trig_ptr->res_id)
 		trig_ptr->orig_res_id = xstrdup(trig_ptr->res_id);
 	trig_ptr->orig_time = trig_ptr->trig_time;
-	
+
 	slurm_mutex_lock(&trigger_mutex);
 	if (trigger_list == NULL)
 		trigger_list = list_create(_trig_del);
@@ -942,11 +933,8 @@ extern int trigger_state_restore(void)
 	if (ver_str) {
 		if (!strcmp(ver_str, TRIGGER_STATE_VERSION)) {
 			protocol_version = SLURM_PROTOCOL_VERSION;
-		} else if (!strcmp(ver_str, TRIGGER_2_2_STATE_VERSION)) {
-			protocol_version = SLURM_2_2_PROTOCOL_VERSION;
-		} else if (!strcmp(ver_str, TRIGGER_2_1_STATE_VERSION)) {
-			protocol_version = SLURM_2_1_PROTOCOL_VERSION;
-		}
+		} else if (!strcmp(ver_str, TRIGGER_2_3_STATE_VERSION))
+			protocol_version = SLURM_2_3_PROTOCOL_VERSION;
 	}
 
 	if (protocol_version == (uint16_t) NO_VAL) {
