@@ -38,9 +38,6 @@
 #ifndef _SET_OOMADJ_H
 #define _SET_OOMADJ_H
 
-/* from linux/mm.h */
-#define OOM_DISABLE (-17)
-
 extern int set_oom_adj(int adj);
 
 #endif /* _SET_OOMADJ_H */
