@@ -133,11 +133,7 @@ parse_command_line( int argc, char* argv[] )
 		params.sort = xstrdup(env_val);
 	if ( ( env_val = getenv("SLURM_CLUSTERS") ) ) {
 		if (!(params.clusters = slurmdb_get_info_cluster(env_val))) {
-			error("'%s' can't be reached now, "
-			      "or it is an invalid entry for "
-			      "SLURM_CLUSTERS.  Use 'sacctmgr --list "
-			      "cluster' to see available clusters.",
-			      env_val);
+			print_db_notok(env_val, 1);
 			exit(1);
 		}
 		working_cluster_rec = list_peek(params.clusters);
@@ -188,11 +184,7 @@ parse_command_line( int argc, char* argv[] )
 				list_destroy(params.clusters);
 			if (!(params.clusters =
 			    slurmdb_get_info_cluster(optarg))) {
-				error("'%s' can't be reached now, "
-				      "or it is an invalid entry for "
-				      "--cluster.  Use 'sacctmgr --list "
-				      "cluster' to see available clusters.",
-				      optarg);
+				print_db_notok(optarg, 0);
 				exit(1);
 			}
 			working_cluster_rec = list_peek(params.clusters);
