@@ -85,7 +85,6 @@ int print_jobs_array(job_info_t * jobs, int size, List format)
 {
 	int i = 0;
 	List l;
-	node_info_msg_t *ni = NULL;
 
 	l = list_create(NULL);
 	if (!params.no_header)
@@ -98,9 +97,6 @@ int print_jobs_array(job_info_t * jobs, int size, List format)
 		list_append(l, (void *) &jobs[i]);
 	}
 	sort_jobs_by_start_time (l);
-	if (ni)
-		slurm_free_node_info_msg (ni);
-
 	sort_job_list (l);
 
 	/* Print the jobs of interest */
